# Copyright 2013 - Noorul Islam K M
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from wsme import types as wtypes
import wsmeext.pecan as wsme_pecan

from solum.api.controllers import common_types
from solum.api.controllers.v1 import app
from solum.api.controllers.v1 import assembly
from solum.api.controllers.v1 import component
from solum.api.controllers.v1.datamodel import types as api_types
from solum.api.controllers.v1 import extension
from solum.api.controllers.v1 import infrastructure
from solum.api.controllers.v1 import language_pack
from solum.api.controllers.v1 import operation
from solum.api.controllers.v1 import pipeline
from solum.api.controllers.v1 import plan
from solum.api.controllers.v1 import sensor
from solum.api.controllers.v1 import service
from solum.api.controllers.v1 import trigger
from solum.common import exception
from solum import version


class Platform(api_types.Base):
    """Representation of a Platform.

    The Platform resource is the root level resource that refers
    to all the other resources owned by this tenant.
    """

    implementation_version = wtypes.text
    "Version of the platform."

    plans_uri = common_types.Uri
    "URI to plans."

    assemblies_uri = common_types.Uri
    "URI to assemblies."

    services_uri = common_types.Uri
    "URI to services."

    components_uri = common_types.Uri
    "URI to components."

    extensions_uri = common_types.Uri
    "URI to extensions."

    operations_uri = common_types.Uri
    "URI to operations."

    sensors_uri = common_types.Uri
    "URI to sensors."

    language_packs_uri = common_types.Uri
    "URI to language packs."

    pipelines_uri = common_types.Uri
    "URI to pipelines."

    triggers_uri = common_types.Uri
    "URI to triggers."

    infrastructure_uri = common_types.Uri
    "URI to infrastructure."

    @classmethod
    def sample(cls):
        return cls(uri='http://example.com/v1',
                   name='solum',
                   type='platform',
                   tags=['solid'],
                   project_id='1dae5a09ef2b4d8cbf3594b0eb4f6b94',
                   user_id='55f41cf46df74320b9486a35f5d28a11',
                   description='solum native implementation',
                   implementation_version='2014.1.1',
                   plans_uri='http://example.com:9777/v1/plans',
                   assemblies_uri='http://example.com:9777/v1/assemblies',
                   services_uri='http://example.com:9777/v1/services',
                   components_uri='http://example.com:9777/v1/components',
                   extensions_uri='http://example.com:9777/v1/extensions',
                   operations_uri='http://example.com:9777/v1/operations',
                   sensors_uri='http://example.com:9777/v1/sensors',
                   language_packs_uri=(
                       'http://example.com:9777/v1/language_packs'),
                   pipelines_uri='http://example.com:9777/v1/pipelines',
                   triggers_uri='http://example.com:9777/v1/triggers',
                   infrastructure_uri=(
                       'http://example.com:9777/v1/infrastructure'))


class Controller(object):
    """Version 1 API controller root."""

    plans = plan.PlansController()
    apps = app.AppsController()
    assemblies = assembly.AssembliesController()
    services = service.ServicesController()
    components = component.ComponentsController()
    extensions = extension.ExtensionsController()
    operations = operation.OperationsController()
    sensors = sensor.SensorsController()
    language_packs = language_pack.LanguagePacksController()
    pipelines = pipeline.PipelinesController()
    triggers = trigger.TriggerController()
    infrastructure = infrastructure.InfrastructureController()

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose(Platform)
    def index(self):
        base_url = pecan.request.application_url.rstrip('/')
        host_url = '%s/%s' % (base_url, 'v1')
        return Platform(uri=host_url,
                        name='solum',
                        type='platform',
                        description='solum native implementation',
                        implementation_version=version.version_string(),
                        plans_uri='%s/plans' % host_url,
                        apps_uri='%s/apps' % host_url,
                        assemblies_uri='%s/assemblies' % host_url,
                        services_uri='%s/services' % host_url,
                        components_uri='%s/components' % host_url,
                        extensions_uri='%s/extensions' % host_url,
                        operations_uri='%s/operations' % host_url,
                        sensors_uri='%s/sensors' % host_url,
                        language_packs_uri='%s/language_packs' % host_url,
                        pipelines_uri='%s/pipelines' % host_url,
                        triggers_uri='%s/triggers' % host_url,
                        infrastructure_uri='%s/infrastructure' % host_url)



# Copyright 2014 - Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""add username to assembly table

Revision ID: 269f9da4b38e
Revises: 586c15b92842
Create Date: 2014-11-14 20:48:25.457672

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '269f9da4b38e'
down_revision = '586c15b92842'


def upgrade():
    op.add_column('assembly',
                  sa.Column('username', sa.String(length=256)))


def downgrade():
    op.drop_column('assembly', 'username')



# Copyright 2014 - Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import string

import wsme
from wsme import types as wtypes

from solum.api.controllers.v1.datamodel import types as api_types
from solum.i18n import _
from solum import objects

TAGS = (TYPE, COMPILER_VERSION, RUNTIME_VERSION, IMPLEMENTATION,
        BUILD_TOOL, OS_PLATFORM, ATTRIBUTE, DESCRIPTION) = (
            'solum::lp::type::', 'solum::lp::compiler_version::',
            'solum::lp::runtime_version::', 'solum::lp::implementation::',
            'solum::lp::build_tool::', 'solum::lp::os_platform::',
            'solum::lp::attribute::', 'solum::lp::description::')


STATE_KIND = wtypes.Enum(str, *objects.image.States.values())
IMAGE_KIND = wtypes.Enum(str, 'auto', 'qcow2', 'docker')
SOURCE_KIND = wtypes.Enum(str, 'auto', 'heroku',
                          'dib', 'dockerfile')


class BuildTool(wtypes.Base):
    """A build tool representation."""

    type = str
    "The type of the build tool."

    version = wtypes.text
    "Version of the build tool."

    @classmethod
    def sample(cls):
        return cls(type='ant', version='1.7')


class LanguagePack(api_types.Base):
    """Representation of a language pack.

    When a user creates an application, he specifies the language pack
    to be used. The language pack is responsible for building the application
    and producing an artifact for deployment.

    For a complete list of language pack attributes please
    refer: https://etherpad.openstack.org/p/Solum-Language-pack-json-format
    """

    def __init__(self, **kwds):
        self.__name = wsme.Unset
        super(LanguagePack, self).__init__(**kwds)

    def get_name(self):
        return self.__name

    def set_name(self, value):
        if len(value) > 100:
            raise ValueError(_('Names must not be longer than 100 '
                               'characters'))
        allowed_chars = string.ascii_lowercase + string.digits + '-_'
        for ch in value:
            if ch not in allowed_chars:
                raise ValueError(_('Names must only contain a-z,0-9,-,_'))
        self.__name = value

    name = wtypes.wsproperty(str, get_name, set_name, mandatory=True)

    language_pack_type = wtypes.text
    """Type of the language pack. Identifies the language supported by the
    language pack. This attribute value will use the
    org.openstack.solum namespace.
    """

    compiler_versions = [wtypes.text]
    """List of all the compiler versions supported by the language pack.
    Example: For a java language pack supporting Java versions 1.4 to 1.7,
    version = ['1.4', '1.6', '1.7']
    """

    runtime_versions = [wtypes.text]
    """List of all runtime versions supported by the language pack.
    Runtime version can be different than compiler version.
    Example: An application can be compiled with java 1.7 but it should
    run in java 1.6 as it is backward compatible.
    """

    language_implementation = wtypes.text
    """Actual language implementation supported by the language pack.
    Example: In case of java it might be 'Sun' or 'openJava'
    In case of C++ it can be 'gcc' or 'icc' or 'microsoft'.
    """

    build_tool_chain = [BuildTool]
    """Toolchain available in the language pack.
    Example: For a java language pack which supports Ant and Maven,
    build_tool_chain = ["{type:ant,version:1.7}","{type:maven,version:1.2}"]
    """

    os_platform = {str: wtypes.text}
    """OS and its version used by the language pack.
    This attribute identifies the base image of the language pack.
    """

    attributes = {str: wtypes.text}
    """Additional section attributes will be used to expose custom
    attributes designed by language pack creator.
    """

    source_uri = wtypes.text
    """The URI of the app/element."""

    source_format = SOURCE_KIND
    """The source repository format."""

    status = STATE_KIND
    """The state of the image. """

    base_image_id = wtypes.text
    """The id (in glance) of the image to customize."""

    image_format = IMAGE_KIND
    """The image format."""

    created_image_id = wtypes.text
    """The id of the created image in glance."""

    lp_metadata = wtypes.text
    """The languagepack meta data."""

    """Parameters that can be used as part of lp building process."""
    lp_params = wtypes.DictType(
        wtypes.text,
        wtypes.DictType(wtypes.text,
                        api_types.MultiType(
                            wtypes.text,
                            int,
                            bool,
                            float)))

    @classmethod
    def from_image(cls, image, host_url):
        as_dict = {}
        image_id = image['id']
        as_dict['uuid'] = image_id
        as_dict['name'] = image['name']
        as_dict['type'] = 'language_pack'
        as_dict['uri'] = '%s/v1/%s/%s' % (host_url, 'language_packs', image_id)
        image_tags = image['tags']
        comp_versions = []
        run_versions = []
        build_tools = []
        attrs = {}
        for tag in image_tags:
            if tag.startswith(DESCRIPTION):
                as_dict['description'] = tag[len(DESCRIPTION):]
            if tag.startswith(TYPE):
                as_dict['language_pack_type'] = tag[len(TYPE):]
            if tag.startswith(COMPILER_VERSION):
                comp_versions.append(tag[len(COMPILER_VERSION):])
            if tag.startswith(RUNTIME_VERSION):
                run_versions.append(tag[len(RUNTIME_VERSION):])
            if tag.startswith(IMPLEMENTATION):
                as_dict['language_implementation'] = tag[len(IMPLEMENTATION):]
            if tag.startswith(BUILD_TOOL):
                bt_type, bt_version = tag[len(BUILD_TOOL):].split('::')
                build_tool = BuildTool(type=bt_type, version=bt_version)
                build_tools.append(build_tool)
            if tag.startswith(OS_PLATFORM):
                osp_type, osp_version = tag[len(OS_PLATFORM):].split('::')
                os_platform = {'OS': osp_type, 'version': osp_version}
                as_dict['os_platform'] = os_platform
            if tag.startswith(ATTRIBUTE):
                key, value = tag[len(ATTRIBUTE):].split('::')
                attrs[key] = value
        as_dict['attributes'] = attrs
        as_dict['compiler_versions'] = comp_versions
        as_dict['runtime_versions'] = run_versions
        as_dict['build_tool_chain'] = build_tools
        return cls(**(as_dict))

    def as_image_dict(self):
        tags = ['solum::lp']
        if self.description is not wsme.Unset:
            tags.append(DESCRIPTION + self.description)
        if self.language_pack_type is not wsme.Unset:
            tags.append(TYPE + self.language_pack_type)
        if self.compiler_versions is not wsme.Unset:
            for cv in self.compiler_versions:
                tags.append(COMPILER_VERSION + cv)
        if self.runtime_versions is not wsme.Unset:
            for rv in self.runtime_versions:
                tags.append(RUNTIME_VERSION + rv)
        if self.language_implementation is not wsme.Unset:
            tags.append(IMPLEMENTATION + self.language_implementation)
        if self.build_tool_chain is not wsme.Unset:
            for bt in self.build_tool_chain:
                tags.append(BUILD_TOOL + bt.type + '::' + bt.version)
        ptfm = self.os_platform
        if ptfm is not wsme.Unset and 'OS' in ptfm and 'version' in ptfm:
            tags.append(OS_PLATFORM + ptfm['OS'] + '::' + ptfm['version'])
        if self.build_tool_chain is not wsme.Unset:
            for key, value in self.attributes.items():
                tags.append(ATTRIBUTE + key + '::' + value)
        # TODO(julienvey) parse specific attributes for image creation from
        # self.attributes, such as image_format...
        return {'name': self.name, 'tags': tags}

    @classmethod
    def sample(cls):
        return cls(uri='http://example.com/v1/images/b3e0d79',
                   source_uri='git://example.com/project/app.git',
                   source_format='heroku',
                   name='php-web-app',
                   type='languagepack',
                   description='A php web application',
                   tags=['group_xyz'],
                   project_id='1dae5a09ef2b4d8cbf3594b0eb4f6b94',
                   user_id='55f41cf46df74320b9486a35f5d28a11',
                   base_image_id='4dae5a09ef2b4d8cbf3594b0eb4f6b94',
                   created_image_id='4afasa09ef2b4d8cbf3594b0ec4f6b94',
                   image_format='docker',
                   language_pack_name='java-1.4-1.7',
                   language_pack_type='org.openstack.solum.Java',
                   language_pack_id='123456789abcdef',
                   compiler_versions=['1.4', '1.6', '1.7'],
                   runtime_versions=['1.4', '1.6', '1.7'],
                   language_implementation='Sun',
                   build_tool_chain=[BuildTool(type='ant', version='1.7'),
                                     BuildTool(type='maven', version='1.2')],
                   os_platform={'OS': 'Ubuntu', 'version': '12.04'},
                   attributes={'optional_attr1': 'value',
                               'admin_email': 'someadmin@somewhere.com'},
                   )



#
# Copyright 2013 - Noorul Islam K M
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

from oslo_config import cfg
import oslo_middleware.cors as cors_middleware
import pecan

from solum.api import auth
from solum.api import config as api_config

import eventlet

if os.name == 'nt':
    # eventlet monkey patching causes subprocess.Popen to fail on Windows
    # when using pipes due to missing non blocking I/O support
    eventlet.monkey_patch(os=False)
else:
    eventlet.monkey_patch()


# Register options for the service
API_SERVICE_OPTS = [
    cfg.IntOpt('port',
               default=9777,
               help='The port for the solum API server'),
    cfg.HostAddressOpt('host',
                       default='127.0.0.1',
                       help='The listen IP for the solum API server'),
    cfg.IntOpt('max_apps_per_tenant',
               default=10,
               help='Maximum number of application allowed per tenant'),
]

API_PLAN_OPTS = [
    cfg.IntOpt('max_plan_size',
               default=65536,
               help='Maximum raw byte size of a plan'),
]


CONF = cfg.CONF
opt_group = cfg.OptGroup(name='api',
                         title='Options for the solum-api service')


def list_opts():
    yield 'api', API_SERVICE_OPTS
    yield None, API_PLAN_OPTS


CONF.register_group(opt_group)
CONF.register_opts(API_SERVICE_OPTS, opt_group)
CONF.register_opts(API_PLAN_OPTS)


def get_pecan_config():
    # Set up the pecan configuration
    filename = api_config.__file__.replace('.pyc', '.py')
    return pecan.configuration.conf_from_file(filename)


def setup_app(config=None):

    if not config:
        config = get_pecan_config()

    app_conf = dict(config.app)

    app = pecan.make_app(
        app_conf.pop('root'),
        logging=getattr(config, 'logging', {}),
        **app_conf
    )

    app = auth.install(app, CONF)

    # Create a CORS wrapper, and attach solum-specific defaults that must be
    # supported on all CORS responses.
    app = cors_middleware.CORS(app, CONF)

    return app



# Copyright 2014 - Rackspace US, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from wsme import types as wtypes

from solum.api.controllers.v1.datamodel import types as api_types


class Pipeline(api_types.Base):
    """Representation of an Pipeline.

    A pipeline is the association between a plan, a mistral workbook
    and a git trigger. Together they form a working development "pipeline".
    """

    plan_uri = wtypes.text
    """Link to the plan URI."""

    workbook_name = wtypes.text
    """Name of the workbook in Mistral to use."""

    trigger_uri = wtypes.text
    """The trigger uri used to trigger the pipeline."""

    last_execution = wtypes.text
    """The UUID of the last run execution."""

    @classmethod
    def from_db_model(cls, m, host_url):
        obj = super(Pipeline, cls).from_db_model(m, host_url)
        le_obj = m.last_execution()
        if le_obj is not None:
            obj.last_execution = le_obj.uuid
        obj.plan_uri = '%s/v1/plans/%s' % (host_url, m.plan_uuid)
        obj.trigger_uri = '%s/v1/triggers/%s' % (host_url, m.trigger_id)
        return obj

    @classmethod
    def sample(cls):
        return cls(uri='http://example.com/v2/pipelines/p1',
                   type='pipeline',
                   name='Example-pipeline',
                   description='A pipeline for my app',
                   trigger_uri='http://example.com/v1/triggers/1abc234',
                   last_execution='78f41cf46df7430b9486a35f5d28a41',
                   tags=['small'],
                   project_id='1dae5a09ef2b4d8cbf3594b0eb4f6b94',
                   user_id='55f41cf46df74320b9486a35f5d28a11',
                   plan_uri='http://example.com/v1/plans/x1',
                   workbook_name='build-deploy')



# Copyright 2014 - Rackspace Hosting
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from solum.deployer.handlers import noop as noop_handler
from solum.i18n import _
from solum.tests import base
from solum.tests import fakes
from solum.tests import utils


class HandlerTest(base.BaseTestCase):
    def setUp(self):
        super(HandlerTest, self).setUp()
        self.ctx = utils.dummy_context()

    @mock.patch('solum.deployer.handlers.noop.LOG')
    def test_echo(self, fake_LOG):
        noop_handler.Handler().echo({}, 'foo')
        fake_LOG.debug.assert_called_once_with(_('%s') % 'foo')

    @mock.patch('solum.deployer.handlers.noop.LOG')
    def test_deploy(self, fake_LOG):
        args = [77, 'created_image_id', [80]]
        noop_handler.Handler().deploy(self.ctx, *args)
        message = 'Deploy %s %s %s' % tuple(args)
        fake_LOG.debug.assert_called_once_with(_("%s") % message)

    @mock.patch('solum.objects.registry')
    @mock.patch('solum.deployer.handlers.noop.LOG')
    def test_destroy(self, fake_LOG, fake_registry):
        fake_assembly = fakes.FakeAssembly()
        fake_registry.Assembly.get_by_id.return_value = fake_assembly
        args = [fake_assembly.id]
        noop_handler.Handler().destroy_assembly(self.ctx, *args)
        fake_assembly.destroy.assert_called_once_with(self.ctx)

        message = 'Destroy %s' % tuple(args)
        fake_LOG.debug.assert_called_once_with(_("%s") % message)



# Copyright 2014 - Rackspace Hosting
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
from unittest import mock

from heatclient import exc
from oslo_config import cfg
import yaml

from solum.common import exception
from solum.deployer.handlers import heat as heat_handler
from solum.objects import assembly
from solum.tests import base
from solum.tests import fakes
from solum.tests import utils


STATES = assembly.States


class HandlerTest(base.BaseTestCase):
    def setUp(self):
        super(HandlerTest, self).setUp()
        self.ctx = utils.dummy_context()

    def test_create(self):
        handler = heat_handler.Handler()
        handler.echo = mock.MagicMock()
        handler.echo({}, 'foo')
        handler.echo.assert_called_once_with({}, 'foo')

    @mock.patch('solum.deployer.handlers.heat.tlog')
    @mock.patch('solum.deployer.handlers.heat.update_assembly')
    @mock.patch('solum.common.catalog.get')
    @mock.patch('solum.objects.registry')
    @mock.patch('solum.common.clients.OpenStackClients')
    @mock.patch('solum.deployer.handlers.heat.get_app_by_assem_id')
    @mock.patch('solum.deployer.handlers.heat.get_heat_client')
    def test_deploy_docker_on_vm_with_dreg(self, mock_ht_cl,
                                           mock_get_app, mock_clients,
                                           mock_registry, mock_get_templ,
                                           mock_ua, m_log):
        handler = heat_handler.Handler()

        fake_assembly = fakes.FakeAssembly()

        m_log.TenantLogger.call.return_value = mock.MagicMock()

        mock_registry.Assembly.get_by_id.return_value = fake_assembly
        fake_template = self._get_fake_template()
        template = self._get_tmpl_for_docker_reg(fake_assembly, fake_template,
                                                 'created_image_id')
        cfg.CONF.api.image_format = "vm"
        cfg.CONF.worker.image_storage = "docker_registry"
        mock_get_templ.return_value = template
        handler._find_id_if_stack_exists = mock.MagicMock(return_value=(None))

        created_stack = {"stack": {
            "id": "fake_id",
            "links": [{"href": "http://fake.ref",
                       "rel": "self"}]}}

        mock_ht_cl.return_value.stacks.create.return_value = created_stack
        handler._check_stack_status = mock.MagicMock()

        handler.deploy(self.ctx, 77, 'created_image_id', 'image_name', [80])
        self.assertTrue(mock_ht_cl.return_value.stacks.create.called)
        assign_and_create_mock = mock_registry.Component.assign_and_create
        comp_name = 'Heat_Stack_for_%s' % fake_assembly.name
        assign_and_create_mock.assert_called_once_with(self.ctx,
                                                       fake_assembly,
                                                       comp_name,
                                                       'heat_stack',
                                                       'Heat Stack test',
                                                       'http://fake.ref',
                                                       'fake_id')

    @mock.patch('solum.deployer.handlers.heat.tlog')
    @mock.patch('solum.deployer.handlers.heat.update_assembly')
    @mock.patch('solum.common.catalog.get')
    @mock.patch('solum.common.catalog.get_from_contrib')
    @mock.patch('solum.objects.registry')
    @mock.patch('solum.common.clients.OpenStackClients')
    @mock.patch('solum.deployer.handlers.heat.get_heat_client')
    def test_deploy_docker_on_vm_swift(self, heat_clnt, mock_clients,
                                       mock_registry,
                                       mock_contrib, mock_get_templ,
                                       mock_ua, m_log):
        handler = heat_handler.Handler()

        mock_log = m_log.TenantLogger.return_value.log
        fake_assembly = fakes.FakeAssembly()
        mock_registry.Assembly.get_by_id.return_value = fake_assembly
        fake_template = self._get_fake_template()
        img = "http://a.b.c/d?temp_url_sig=v&temp_url_expires=v"

        mock_contrib.return_value = "robust_file"
        get_file_dict = {}
        get_file_dict[self._get_key()] = "robust_file"

        cfg.CONF.api.image_format = "vm"
        cfg.CONF.worker.image_storage = "swift"
        cfg.CONF.deployer.flavor = "flavor"
        cfg.CONF.deployer.image = "coreos"
        mock_get_templ.return_value = fake_template
        handler._find_id_if_stack_exists = mock.MagicMock(return_value=(None))

        created_stack = {"stack": {
            "id": "fake_id",
            "links": [{"href": "http://fake.ref",
                       "rel": "self"}]}}
        heat_clnt.return_value.stacks.create.return_value = created_stack

        handler._check_stack_status = mock.MagicMock()

        handler.deploy(self.ctx, 77, img, 'tenant-name-ts-commit', [80])

        parameters = {'name': fake_assembly.uuid,
                      'flavor': "flavor",
                      'image': "coreos",
                      'key_name': "mykey",
                      'location': img,
                      'du': 'tenant-name-ts-commit',
                      'publish_ports': '-p 80:80'}

        heat_clnt.return_value.stacks.create.assert_called_once_with(
            stack_name='faker-test_uuid',
            template=fake_template,
            parameters=parameters,
            files=get_file_dict)
        assign_and_create_mock = mock_registry.Component.assign_and_create
        comp_name = 'Heat_Stack_for_%s' % fake_assembly.name
        assign_and_create_mock.assert_called_once_with(self.ctx,
                                                       fake_assembly,
                                                       comp_name,
                                                       'heat_stack',
                                                       'Heat Stack test',
                                                       'http://fake.ref',
                                                       'fake_id')
        self.assertTrue(mock_log.called)

    @mock.patch('solum.deployer.handlers.heat.tlog')
    @mock.patch('solum.deployer.handlers.heat.update_assembly')
    @mock.patch('solum.common.catalog.get')
    @mock.patch('solum.objects.registry')
    @mock.patch('solum.common.clients.OpenStackClients')
    @mock.patch('solum.deployer.handlers.heat.get_heat_client')
    def test_comp_name_error(self, heat_clnt, mock_clients, mock_registry,
                             mock_get_templ, mock_ua, m_log):
        handler = heat_handler.Handler()

        mock_log = m_log.TenantLogger.return_value.log

        fake_assembly = fakes.FakeAssembly()
        mock_registry.Assembly.get_by_id.return_value = fake_assembly
        fake_template = json.dumps({'description': 'test'})
        mock_get_templ.return_value = fake_template
        handler._find_id_if_stack_exists = mock.MagicMock(return_value=(None))
        stacks = heat_clnt.return_value.heat.return_value.stacks
        stacks.create.return_value = {"stack": {
            "id": "fake_id",
            "links": [{"href": "http://fake.ref",
                       "rel": "self"}]}}
        handler._check_stack_status = mock.MagicMock()
        handler.deploy(self.ctx, 77, 'created_image_id', 'image_name', [80])
        assign_and_create_mock = mock_registry.Component.assign_and_create
        comp_name = 'Heat Stack for %s' % fake_assembly.name
        self.assertRaises(AssertionError,
                          assign_and_create_mock.assert_called_once_with,
                          comp_name)
        self.assertTrue(mock_log.called)

    @mock.patch('solum.deployer.handlers.heat.tlog')
    @mock.patch('solum.deployer.handlers.heat.update_assembly')
    @mock.patch('solum.common.catalog.get')
    @mock.patch('solum.common.catalog.get_from_contrib')
    @mock.patch('solum.objects.registry')
    @mock.patch('solum.common.clients.OpenStackClients')
    @mock.patch('solum.deployer.handlers.heat.get_heat_client')
    def test_deploy_docker(self, heat_clnt, mock_clients, mock_registry,
                           mock_get_contrib, mock_get_templ, mock_ua, m_log):
        handler = heat_handler.Handler()
        fake_assembly = fakes.FakeAssembly()
        mock_registry.Assembly.get_by_id.return_value = fake_assembly

        m_log.TenantLogger.call.return_value = mock.MagicMock()

        cfg.CONF.api.image_format = "docker"

        mock_get_contrib.return_value = "robust_file"
        get_file_dict = {}
        get_file_dict[self._get_key()] = "robust_file"

        fake_template = self._get_fake_template()
        template = self._get_tmpl_for_docker_reg(fake_assembly, fake_template,
                                                 'created_image_id')
        mock_get_templ.return_value = template

        handler._find_id_if_stack_exists = mock.MagicMock(return_value=(None))
        created_stack = {"stack": {
            "id": "fake_id",
            "links": [{"href": "http://fake.ref",
                       "rel": "self"}]}}
        heat_clnt.return_value.stacks.create.return_value = created_stack
        handler._check_stack_status = mock.MagicMock()

        handler.deploy(self.ctx, 77, 'created_image_id', 'image_name', [80])

        parameters = {'image': 'created_image_id',
                      'app_name': 'faker',
                      'port': 80}

        heat_clnt.return_value.stacks.create.assert_called_once_with(
            stack_name='faker-test_uuid',
            template=template,
            parameters=parameters,
            files=get_file_dict)
        assign_and_create_mock = mock_registry.Component.assign_and_create
        comp_name = 'Heat_Stack_for_%s' % fake_assembly.name
        assign_and_create_mock.assert_called_once_with(self.ctx,
                                                       fake_assembly,
                                                       comp_name,
                                                       'heat_stack',
                                                       'Heat Stack test',
                                                       'http://fake.ref',
                                                       'fake_id')

    @mock.patch('solum.deployer.handlers.heat.update_wf_and_app')
    @mock.patch('solum.deployer.handlers.heat.update_assembly')
    @mock.patch('solum.common.clients.OpenStackClients')
    @mock.patch('httplib2.Http')
    @mock.patch('solum.common.repo_utils')
    @mock.patch('solum.deployer.handlers.heat.get_heat_client')
    def test_update_assembly_status(self, heat_clnt, mock_repo, mock_http,
                                    mock_clients, mock_ua, mock_wf_app):

        cfg.CONF.set_override('wait_interval', 1, group='deployer')
        cfg.CONF.set_override('growth_factor', 1, group='deployer')
        cfg.CONF.set_override('max_attempts', 1, group='deployer')

        handler = heat_handler.Handler()
        fake_assembly = fakes.FakeAssembly()
        stack = mock.MagicMock()
        stack.status = 'COMPLETE'
        heat_clnt.return_value.stacks.get.return_value = stack

        resp = {'status': '200'}
        conn = mock.MagicMock()
        conn.request.return_value = [resp, '']
        mock_http.return_value = conn

        cfg.CONF.deployer.du_attempts = 1

        mock_logger = mock.MagicMock()
        handler._parse_server_ip = mock.MagicMock(return_value=('xyz'))
        mock_repo.is_reachable.return_value = True
        handler._check_stack_status(self.ctx, fake_assembly.id, heat_clnt,
                                    'fake_id', [80], mock_logger)

        c1 = mock.call(self.ctx, fake_assembly.id,
                       {'status': STATES.STARTING_APP,
                        'application_uri': 'xyz:80'})

        c2 = mock.call(self.ctx, fake_assembly.id,
                       {'status': 'DEPLOYMENT_COMPLETE'})

        calls = [c1, c2]

        mock_ua.assert_has_calls(calls, any_order=False)

    @mock.patch('solum.deployer.handlers.heat.update_wf_and_app')
    @mock.patch('solum.deployer.handlers.heat.update_assembly')
    @mock.patch('solum.common.clients.OpenStackClients')
    @mock.patch('httplib2.Http')
    @mock.patch('solum.deployer.handlers.heat.get_heat_client')
    def test_update_assembly_status_multiple_ports(self, heat_clnt, mock_http,
                                                   mock_clients, mock_ua,
                                                   mock_wf_app):

        cfg.CONF.set_override('wait_interval', 1, group='deployer')
        cfg.CONF.set_override('growth_factor', 1, group='deployer')
        cfg.CONF.set_override('max_attempts', 1, group='deployer')

        handler = heat_handler.Handler()
        fake_assembly = fakes.FakeAssembly()
        stack = mock.MagicMock()
        stack.status = 'COMPLETE'
        heat_clnt.stacks.get.return_value = stack

        resp = {'status': '200'}
        conn = mock.MagicMock()
        conn.request.return_value = [resp, '']
        mock_http.return_value = conn

        cfg.CONF.deployer.du_attempts = 1

        mock_logger = mock.MagicMock()
        handler._parse_server_ip = mock.MagicMock(return_value=('xyz'))
        handler._check_stack_status(self.ctx, fake_assembly.id,
                                    heat_clnt.return_value,
                                    'fake_id', [80, 81], mock_logger)

        c1 = mock.call(self.ctx, fake_assembly.id,
                       {'status': STATES.STARTING_APP,
                        'application_uri': 'xyz:[80,81]'})

        c2 = mock.call(self.ctx, fake_assembly.id,
                       {'status': 'DEPLOYMENT_COMPLETE'})

        calls = [c1, c2]

        mock_ua.assert_has_calls(calls, any_order=False)

    @mock.patch('solum.deployer.handlers.heat.update_assembly')
    @mock.patch('solum.common.clients.OpenStackClients')
    @mock.patch('solum.deployer.handlers.heat.get_heat_client')
    def test_update_assembly_status_failed(self, heat_clnt,
                                           mock_clients, mock_ua):

        cfg.CONF.set_override('wait_interval', 1, group='deployer')
        cfg.CONF.set_override('growth_factor', 1, group='deployer')
        cfg.CONF.set_override('max_attempts', 1, group='deployer')

        handler = heat_handler.Handler()
        fake_assembly = fakes.FakeAssembly()
        stack = mock.MagicMock()
        stack.status = 'FAILED'
        heat_clnt.return_value.stacks.get.return_value = stack
        mock_logger = mock.MagicMock()
        handler._check_stack_status(self.ctx, fake_assembly.id,
                                    heat_clnt.return_value,
                                    'fake_id', [80], mock_logger)
        mock_ua.assert_called_once_with(self.ctx, fake_assembly.id,
                                        {'status':
                                         STATES.ERROR_STACK_CREATE_FAILED})

    @mock.patch('solum.deployer.handlers.heat.update_assembly')
    def test_get_template(self, mua):
        handler = heat_handler.Handler()
        fake_assembly = fakes.FakeAssembly()

        image_format = 'vm'
        image_storage = 'glance'
        image_loc = 'abc'
        image_name = 'def'
        ports = [80]
        mock_logger = mock.MagicMock()
        template = handler._get_template(self.ctx, image_format,
                                         image_storage, image_loc, image_name,
                                         fake_assembly, ports, mock_logger)
        self.assertIsNone(template)
        mua.assert_called_once_with(self.ctx,
                                    fake_assembly.id,
                                    {'status': STATES.ERROR})

    @mock.patch('solum.deployer.handlers.heat.update_assembly')
    def test_get_template_vm_glance(self, mock_update_assembly):
        handler = heat_handler.Handler()
        fake_assembly = fakes.FakeAssembly()

        image_format = 'vm'
        image_storage = 'glance'
        image_loc = 'abc'
        image_name = 'def'
        ports = [80]
        mock_logger = mock.MagicMock()
        template = handler._get_template(self.ctx, image_format,
                                         image_storage, image_loc, image_name,
                                         fake_assembly, ports, mock_logger)
        self.assertIsNone(template)
        mock_update_assembly.assert_called_once_with(self.ctx,
                                                     fake_assembly.id,
                                                     {'status': STATES.ERROR})

    @mock.patch('solum.common.catalog.get')
    def test_get_template_vm_docker_reg(self, mock_catalog_get):
        handler = heat_handler.Handler()
        fake_assembly = fakes.FakeAssembly()

        template_getter = mock.MagicMock()
        template_getter.return_value = self._get_fake_template()
        handler._get_template_for_docker_reg = template_getter

        image_format = 'vm'
        image_storage = 'docker_registry'
        image_loc = 'abc'
        image_name = 'def'
        ports = [80]
        mock_logger = mock.MagicMock()
        template = handler._get_template(self.ctx, image_format,
                                         image_storage, image_loc, image_name,
                                         fake_assembly, ports, mock_logger)
        self.assertIsNotNone(template)
        self.assertTrue(handler._get_template_for_docker_reg.called)
        mock_catalog_get.assert_called_once_with('templates', 'coreos')

    @mock.patch('solum.common.catalog.get')
    def test_get_template_vm_swift(self, mock_catalog_get):
        handler = heat_handler.Handler()
        fake_assembly = fakes.FakeAssembly()
        mock_logger = mock.MagicMock()

        image_format = 'vm'
        image_storage = 'swift'
        image_loc = 'abc'
        image_name = 'def'
        ports = [80]

        mock_catalog_get.return_value = self._get_fake_template()

        template = handler._get_template(self.ctx, image_format,
                                         image_storage, image_loc,
                                         image_name, fake_assembly,
                                         ports, mock_logger)

        self.assertEqual(self._get_fake_template(), template)
        self.assertEqual(1, mock_catalog_get.call_count)
        self.assertEqual(mock.call('templates', 'coreos'),
                         mock_catalog_get.call_args)

    @mock.patch('solum.deployer.handlers.heat.update_assembly')
    @mock.patch('solum.common.catalog.get')
    def test_get_template_vm_swift_error(self, mock_catalog_get, mock_ua):
        handler = heat_handler.Handler()
        fake_assembly = fakes.FakeAssembly()

        exc_obj = exception.ObjectNotFound()
        mock_catalog_get.side_effect = exc_obj

        template_getter = mock.MagicMock()
        template_getter.return_value = self._get_fake_template()
        handler._get_template_for_swift = template_getter

        image_format = 'vm'
        image_storage = 'swift'
        image_loc = 'abc'
        image_name = 'def'
        ports = [80]
        mock_logger = mock.MagicMock()
        template = handler._get_template(self.ctx, image_format,
                                         image_storage, image_loc, image_name,
                                         fake_assembly, ports, mock_logger)
        self.assertIsNone(template)
        mock_ua.assert_called_once_with(self.ctx, fake_assembly.id,
                                        {'status': STATES.ERROR})

        assert not handler._get_template_for_swift.called

    @mock.patch('solum.deployer.handlers.heat.update_assembly')
    @mock.patch('solum.common.catalog.get')
    def test_get_template_docker_read_error(self, mock_catalog_get, mock_ua):
        handler = heat_handler.Handler()
        fake_assembly = fakes.FakeAssembly()

        exc_obj = exception.ObjectNotFound()
        mock_catalog_get.side_effect = exc_obj

        template_getter = mock.MagicMock()
        template_getter.return_value = self._get_fake_template()
        handler._get_template_for_swift = template_getter

        image_format = 'docker'
        image_storage = 'swift'
        image_loc = 'abc'
        image_name = 'def'
        ports = [80]
        mock_logger = mock.MagicMock()
        template = handler._get_template(self.ctx, image_format,
                                         image_storage, image_loc, image_name,
                                         fake_assembly, ports, mock_logger)
        self.assertIsNone(template)
        mock_ua.assert_called_once_with(self.ctx, fake_assembly.id,
                                        {'status': STATES.ERROR})

    @mock.patch('solum.common.heat_utils.get_network_parameters')
    @mock.patch('solum.common.clients.OpenStackClients')
    def test_get_parameters_for_docker(self, mock_clients, mock_heat_utils):

        handler = heat_handler.Handler()
        fake_assembly = fakes.FakeAssembly()

        image_format = 'docker'
        image_loc = 'abc'
        image_name = 'abc'
        ports = [80]

        mock_logger = mock.MagicMock()

        params = handler._get_parameters(self.ctx, image_format,
                                         image_loc, image_name, fake_assembly,
                                         ports, mock_clients, mock_logger)

        self.assertEqual(fake_assembly.name, params['app_name'])
        self.assertEqual('abc', params['image'])
        self.assertEqual(80, params['port'])

    @mock.patch('solum.common.heat_utils.get_network_parameters')
    @mock.patch('solum.common.clients.OpenStackClients')
    def test_get_parameters_for_vm(self, mock_clients, mock_heat_utils):

        handler = heat_handler.Handler()
        fake_assembly = fakes.FakeAssembly()

        image_format = 'vm'
        image_loc = 'abc'
        image_name = 'abc'
        ports = [80]

        mock_logger = mock.MagicMock()

        cfg.CONF.set_override('flavor', 'abc', group='deployer')
        cfg.CONF.set_override('image', 'def', group='deployer')

        params = handler._get_parameters(self.ctx, image_format,
                                         image_loc, image_name, fake_assembly,
                                         ports, mock_clients, mock_logger)

        self.assertEqual(str(fake_assembly.uuid), params['name'])
        self.assertEqual('abc', params['flavor'])
        self.assertEqual('def', params['image'])
        self.assertIsNone(params.get('port'))

    @mock.patch('solum.deployer.handlers.heat.update_assembly')
    @mock.patch('solum.common.heat_utils.get_network_parameters')
    @mock.patch('solum.common.clients.OpenStackClients')
    def test_get_parameters_for_unrecognized_img_format(self, mock_clients,
                                                        mock_heat_utils,
                                                        mock_ua):

        handler = heat_handler.Handler()
        fake_assembly = fakes.FakeAssembly()

        image_format = 'abc'
        image_loc = 'abc'
        image_name = 'abc'
        ports = [80]

        mock_logger = mock.MagicMock()

        params = handler._get_parameters(self.ctx, image_format,
                                         image_loc, image_name, fake_assembly,
                                         ports, mock_clients, mock_logger)

        self.assertIsNone(params)

        mock_ua.assert_called_once_with(self.ctx, fake_assembly.id,
                                        {'status':
                                         STATES.ERROR})

    @mock.patch('solum.deployer.handlers.heat.update_assembly')
    def test_check_stack_status(self, mock_ua):
        handler = heat_handler.Handler()
        fake_assembly = fakes.FakeAssembly()

        mock_client = mock.MagicMock()
        mock_client.stacks.get.side_effect = Exception()

        cfg.CONF.set_override('wait_interval', 1, group='deployer')
        cfg.CONF.set_override('growth_factor', 1, group='deployer')
        cfg.CONF.set_override('max_attempts', 1, group='deployer')

        mock_logger = mock.MagicMock()
        handler._check_stack_status(self.ctx, fake_assembly.id, mock_client,
                                    'fake_id', [80], mock_logger)
        mock_ua.assert_called_once_with(self.ctx, fake_assembly.id,
                                        {'status':
                                         STATES.ERROR_STACK_CREATE_FAILED})

    def test_parse_server_ip(self):
        handler = heat_handler.Handler()
        heat_output = mock.MagicMock()
        heat_output._info = {"id": "fake_id",
                             "outputs": [{"output_value": "192.168.78.21",
                                          "description": "IP", "output_key":
                                          "public_ip"},
                                         {"output_value":
                                          "http://192.168.78.21:5000",
                                          "description": "URL", "output_key":
                                          "URL"}]}
        host_url = handler._parse_server_ip(heat_output)
        self.assertEqual("192.168.78.21", host_url)

    def test_find_id_if_stack_exists(self):
        handler = heat_handler.Handler()
        assem = mock.MagicMock
        assem.heat_stack_component = mock.MagicMock
        assem.heat_stack_component.heat_stack_id = '123'
        id = handler._find_id_if_stack_exists(assem)
        self.assertEqual('123', id)

    @mock.patch('solum.common.solum_swiftclient.SwiftClient.delete_object')
    @mock.patch('solum.objects.registry')
    @mock.patch('solum.deployer.handlers.heat.tlog')
    @mock.patch('solum.api.handlers.userlog_handler.UserlogHandler')
    def test_delete_app_artifacts_from_swift(self, mock_log_handler, m_log,
                                             mock_registry, mock_swift_delete):
        fake_assembly = fakes.FakeAssembly()
        fake_image = fakes.FakeImage()
        mock_registry.Image.get_by_id.return_value = fake_image
        handler = heat_handler.Handler()
        handler._delete_app_artifacts_from_swift(self.ctx, mock_log_handler,
                                                 fake_assembly)
        mock_registry.Image.get_by_id.assert_called_once_with(
            mock.ANY, fake_assembly.image_id)
        docker_image_name = fake_image.docker_image_name
        img_filename = docker_image_name.split('-', 1)[1]
        mock_swift_delete.assert_called_once_with('solum_du', img_filename)

    @mock.patch('solum.common.solum_swiftclient.SwiftClient.delete_object')
    @mock.patch('solum.objects.registry')
    @mock.patch('solum.deployer.handlers.heat.tlog')
    @mock.patch('solum.api.handlers.userlog_handler.UserlogHandler')
    def test_delete_app_artifacts_from_swift_no_image(self, mock_log_handler,
                                                      m_log, mock_registry,
                                                      mock_swift_delete):
        fake_assembly = fakes.FakeAssembly()
        fake_assembly.image_id = None
        handler = heat_handler.Handler()
        handler._delete_app_artifacts_from_swift(self.ctx, mock_log_handler,
                                                 fake_assembly)
        self.assertFalse(mock_registry.Image.get_by_id.called)
        self.assertFalse(mock_swift_delete.called)

    @mock.patch('solum.common.solum_swiftclient.SwiftClient.delete_object')
    @mock.patch('solum.api.handlers.userlog_handler.UserlogHandler')
    @mock.patch('solum.deployer.handlers.heat.tlog')
    @mock.patch('solum.objects.registry')
    @mock.patch('solum.common.clients.OpenStackClients')
    @mock.patch('solum.deployer.handlers.heat.get_heat_client')
    def test_destroy_success(self, heat_clnt, mock_client, mock_registry,
                             m_log, mock_log_handler, mock_swift_delete):
        cfg.CONF.set_override('image_storage', 'swift', group='worker')
        fake_assem = fakes.FakeAssembly()
        mock_registry.Assembly.get_by_id.return_value = fake_assem
        fake_image = fakes.FakeImage()
        mock_registry.Image.get_by_id.return_value = fake_image

        handler = heat_handler.Handler()

        mock_log = m_log.TenantLogger.return_value.log

        mock_del = heat_clnt.return_value.stacks.delete
        mock_del.side_effect = exc.HTTPNotFound

        cfg.CONF.deployer.max_attempts = 1
        cfg.CONF.deployer.wait_interval = 0
        cfg.CONF.deployer.growth_factor = 1.2

        with mock.patch.object(handler, "_find_id_if_stack_exists",
                               return_value=42):

            handler.destroy_assembly(self.ctx, fake_assem.id)

            self.assertTrue(mock_del.called)
            mock_registry.Image.get_by_id.assert_called_once_with(
                mock.ANY, fake_assem.image_id)
            docker_image_name = fake_image.docker_image_name
            img_filename = docker_image_name.split('-', 1)[1]
            mock_swift_delete.assert_called_once_with('solum_du', img_filename)
            log_handler = mock_log_handler.return_value
            log_handler.delete.assert_called_once_with(fake_assem.uuid)
            self.assertTrue(mock_log.called)

    @mock.patch('solum.common.solum_swiftclient.SwiftClient.delete_object')
    @mock.patch('solum.api.handlers.userlog_handler.UserlogHandler')
    @mock.patch('solum.deployer.handlers.heat.tlog')
    @mock.patch('solum.objects.registry')
    @mock.patch('solum.common.clients.OpenStackClients')
    @mock.patch('solum.deployer.handlers.heat.get_heat_client')
    def test_destroy_stack_not_found(self, heat_clnt, mock_client,
                                     mock_registry, m_log,
                                     mock_log_handler, mock_swift_delete):
        cfg.CONF.set_override('image_storage', 'swift', group='worker')
        fake_assem = fakes.FakeAssembly()
        mock_registry.Assembly.get_by_id.return_value = fake_assem
        fake_image = fakes.FakeImage()
        mock_registry.Image.get_by_id.return_value = fake_image

        handler = heat_handler.Handler()

        mock_log = m_log.TenantLogger.return_value.log

        mock_del = heat_clnt.return_value.stacks.delete
        mock_del.side_effect = exc.HTTPNotFound

        cfg.CONF.deployer.max_attempts = 1
        cfg.CONF.deployer.wait_interval = 0
        cfg.CONF.deployer.growth_factor = 1.2

        with mock.patch.object(handler, "_find_id_if_stack_exists",
                               return_value=42) as mock_find:

            handler.destroy_assembly(self.ctx, fake_assem.id)

            self.assertTrue(mock_find.called)
            self.assertTrue(mock_del.called)
            mock_registry.Image.get_by_id.assert_called_once_with(
                mock.ANY, fake_assem.image_id)
            docker_image_name = fake_image.docker_image_name
            img_filename = docker_image_name.split('-', 1)[1]
            mock_swift_delete.assert_called_once_with('solum_du', img_filename)
            log_handler = mock_log_handler.return_value
            log_handler.delete.assert_called_once_with(fake_assem.uuid)
            self.assertTrue(mock_log.called)

    @mock.patch('solum.deployer.handlers.heat.tlog')
    @mock.patch('solum.deployer.handlers.heat.update_assembly')
    @mock.patch('solum.objects.registry')
    @mock.patch('solum.common.clients.OpenStackClients')
    @mock.patch('solum.deployer.handlers.heat.get_heat_client')
    def test_destroy_error(self, heat_clnt, mock_client,
                           mock_registry, mua, m_log):
        fake_assem = fakes.FakeAssembly()
        mock_registry.Assembly.get_by_id.return_value = fake_assem

        mock_del = heat_clnt.return_value.stacks.delete

        handler = heat_handler.Handler()
        handler._find_id_if_stack_exists = mock.MagicMock(return_value='42')

        cfg.CONF.deployer.max_attempts = 1
        cfg.CONF.deployer.wait_interval = 0
        cfg.CONF.deployer.growth_factor = 1.2

        with mock.patch.object(handler, "_find_id_if_stack_exists",
                               return_value="42"):

            handler.destroy_assembly(self.ctx, fake_assem.id)

            c1 = mock.call(self.ctx, fake_assem.id,
                           {'status': STATES.DELETING})

            c2 = mock.call(self.ctx, fake_assem.id,
                           {'status': STATES.ERROR_STACK_DELETE_FAILED})

            calls = [c1, c2]

            mua.assert_has_calls(calls, any_order=False)

        self.assertTrue(mock_del.called)

    @mock.patch('solum.common.solum_swiftclient.SwiftClient.delete_object')
    @mock.patch('solum.api.handlers.userlog_handler.UserlogHandler')
    @mock.patch('solum.deployer.handlers.heat.tlog')
    @mock.patch('solum.objects.registry')
    @mock.patch('solum.common.clients.OpenStackClients')
    def test_destroy_absent_no_wf(self, mock_client, mock_registry,
                                  mock_tlogger, mock_log_handler,
                                  mock_swift_delete):

        cfg.CONF.worker.image_storage = "swift"
        fake_assem = fakes.FakeAssembly()
        mock_registry.Assembly.get_by_id.return_value = fake_assem
        fake_image = fakes.FakeImage()
        mock_registry.Image.get_by_id.return_value = fake_image

        nfe = exception.ResourceNotFound(name=fake_assem.name,
                                         id=fake_assem.id)
        mock_registry.Workflow.get_by_assembly_id.side_effect = nfe

        mock_log = mock_tlogger.TenantLogger.return_value.log
        mock_upload = mock_tlogger.TenantLogger.return_value.upload
        mock_log_del = mock_log_handler.return_value.delete
        mock_heat = mock_client.return_value.heat
        mock_del = mock_heat.return_value.stacks.delete

        hh = heat_handler.Handler()
        with mock.patch.object(hh, "_find_id_if_stack_exists",
                               return_value=None):

            hh.destroy_assembly(self.ctx, fake_assem.id)

        self.assertFalse(mock_del.called)
        self.assertTrue(fake_assem.destroy.called)
        mock_registry.Image.get_by_id.assert_called_once_with(
            mock.ANY, fake_assem.image_id)
        docker_image_name = fake_image.docker_image_name
        img_filename = docker_image_name.split('-', 1)[1]
        mock_swift_delete.assert_called_once_with('solum_du', img_filename)
        self.assertTrue(mock_log.called)
        self.assertTrue(mock_upload.called)
        self.assertEqual(1, mock_log_del.call_count)
        self.assertEqual(mock.call(fake_assem.uuid), mock_log_del.call_args)

    @mock.patch('solum.common.solum_swiftclient.SwiftClient.delete_object')
    @mock.patch('solum.api.handlers.userlog_handler.UserlogHandler')
    @mock.patch('solum.deployer.handlers.heat.tlog')
    @mock.patch('solum.objects.registry')
    @mock.patch('solum.common.clients.OpenStackClients')
    def test_destroy_absent_wf_present(self, mock_client, mock_registry,
                                       mock_tlogger, mock_log_handler,
                                       mock_swift_delete):

        cfg.CONF.worker.image_storage = "swift"
        fake_assem = fakes.FakeAssembly()
        mock_registry.Assembly.get_by_id.return_value = fake_assem
        fake_image = fakes.FakeImage()
        mock_registry.Image.get_by_id.return_value = fake_image

        fake_wf = fakes.FakeWorkflow()
        mock_registry.Workflow.get_by_assembly_id.side_effect = fake_wf

        mock_log = mock_tlogger.TenantLogger.return_value.log
        mock_upload = mock_tlogger.TenantLogger.return_value.upload
        mock_log_del = mock_log_handler.return_value.delete
        mock_heat = mock_client.return_value.heat
        mock_del = mock_heat.return_value.stacks.delete

        hh = heat_handler.Handler()
        with mock.patch.object(hh, "_find_id_if_stack_exists",
                               return_value=None):

            hh.destroy_assembly(self.ctx, fake_assem.id)

        self.assertFalse(mock_del.called)
        self.assertFalse(fake_assem.destroy.called)
        mock_registry.Image.get_by_id.assert_called_once_with(
            mock.ANY, fake_assem.image_id)
        docker_image_name = fake_image.docker_image_name
        img_filename = docker_image_name.split('-', 1)[1]
        mock_swift_delete.assert_called_once_with('solum_du', img_filename)
        self.assertTrue(mock_log.called)
        self.assertTrue(mock_upload.called)
        self.assertEqual(1, mock_log_del.call_count)
        self.assertEqual(mock.call(fake_assem.uuid), mock_log_del.call_args)

    @mock.patch('solum.objects.registry')
    @mock.patch('solum.common.clients.OpenStackClients')
    @mock.patch('solum.deployer.handlers.heat.get_heat_client')
    def test_successful_deploy_destroys_twins(self, heat_clnt,
                                              mock_client, mr):
        handler = heat_handler.Handler()
        old_app = fakes.FakeAssembly()
        old_app.name = 'old app'
        old_app.status = 'DEPLOYMENT_COMPLETE'

        new_app = fakes.FakeAssembly()
        new_app.id = 9
        new_app.plan_id = old_app.plan_id
        new_app.name = 'new app'
        new_app.status = 'DEPLOYMENT_COMPLETE'

        cfg.CONF.set_override('wait_interval', 0, group='deployer')
        cfg.CONF.set_override('growth_factor', 0, group='deployer')
        cfg.CONF.set_override('max_attempts', 1, group='deployer')

        self.assertEqual(old_app.plan_id, new_app.plan_id)
        self.assertEqual(old_app.plan_uuid, new_app.plan_uuid)
        mr.AssemblyList.get_earlier.return_value = [old_app]

        mock_st_del = heat_clnt.return_value.stacks.delete
        mock_st_get = heat_clnt.return_value.stacks.get

        handler.destroy_assembly = mock.MagicMock()
        handler._destroy_other_assemblies(self.ctx, new_app.id,
                                          heat_clnt.return_value)
        self.assertTrue(mock_st_del.called)
        self.assertTrue(mock_st_get.called)

    @mock.patch('solum.objects.registry')
    @mock.patch('solum.common.clients.OpenStackClients')
    @mock.patch('solum.deployer.handlers.heat.get_heat_client')
    def test_successful_deploy_preserves_others(self, heat_clnt,
                                                mock_client, mr):
        handler = heat_handler.Handler()
        old_app = fakes.FakeAssembly()
        old_app.name = 'old app'
        old_app.plan_id = 1
        old_app.id = 1
        old_app.status = 'DEPLOYMENT_COMPLETE'

        new_app = fakes.FakeAssembly()
        new_app.id = 1
        new_app.plan_id = 2
        new_app.plan_uuid = 'new fake plan uuid'
        new_app.name = 'new app'
        new_app.status = 'DEPLOYMENT_COMPLETE'

        cfg.CONF.set_override('wait_interval', 0, group='deployer')
        cfg.CONF.set_override('growth_factor', 0, group='deployer')
        cfg.CONF.set_override('max_attempts', 1, group='deployer')

        self.assertNotEqual(old_app.plan_id, new_app.plan_id)
        self.assertNotEqual(old_app.plan_uuid, new_app.plan_uuid)
        mr.AssemblyList.get_earlier.return_value = [old_app]

        mr.Assembly.get_by_id.return_value = new_app

        mock_heat = mock_client.return_value.heat
        mock_st_del = mock_heat.return_value.stacks.delete
        mock_st_get = mock_heat.return_value.stacks.get

        handler.destroy_assembly = mock.MagicMock()
        handler._destroy_other_assemblies(self.ctx, new_app.id, heat_clnt)
        self.assertFalse(mock_st_del.called)
        self.assertFalse(mock_st_get.called)

    @mock.patch('solum.objects.registry')
    @mock.patch('solum.deployer.handlers.heat.get_heat_client')
    def test_successful_deploy_preserves_notreadies(self, heat_clnt, mr):
        handler = heat_handler.Handler()
        old_app = fakes.FakeAssembly()
        old_app.name = 'old app'
        old_app.status = 'BUILDING'

        new_app = fakes.FakeAssembly()
        new_app.id = 9
        new_app.plan_id = old_app.plan_id
        new_app.name = 'new app'
        new_app.status = 'DEPLOYMENT_COMPLETE'

        self.assertEqual(old_app.plan_id, new_app.plan_id)
        self.assertEqual(old_app.plan_uuid, new_app.plan_uuid)
        mr.AssemblyList.get_earlier.return_value = []

        handler.destroy_assembly = mock.MagicMock()
        handler._destroy_other_assemblies(self.ctx, new_app.id, heat_clnt)
        self.assertEqual(0, handler.destroy_assembly.call_count)

    @mock.patch('solum.objects.registry')
    @mock.patch('solum.deployer.handlers.heat.get_heat_client')
    def test_unsuccessful_deploy_preserves_everyone(self, heat_clnt, mr):
        handler = heat_handler.Handler()
        old_app = fakes.FakeAssembly()
        old_app.name = 'old app'
        old_app.status = 'DEPLOYMENT_COMPLETE'

        new_app = fakes.FakeAssembly()
        new_app.id = 9
        new_app.plan_id = old_app.plan_id
        new_app.name = 'new app'
        new_app.status = 'ERROR'

        self.assertEqual(old_app.plan_id, new_app.plan_id)
        self.assertEqual(old_app.plan_uuid, new_app.plan_uuid)
        mr.AssemblyList.get_earlier.return_value = []

        handler.destroy_assembly = mock.MagicMock()
        handler._destroy_other_assemblies(self.ctx, new_app.id,
                                          heat_clnt.return_value)
        self.assertEqual(0, handler.destroy_assembly.call_count)

    def _get_key(self):
        return "robust-du-handling.sh"

    def _get_fake_template(self):
        t = "description: test\n"
        t += "resources:\n"
        t += "  compute_instance:\n"
        t += "    properties:\n"
        t += "      user_data:\n"
        t += "        str_replace:\n"
        t += "          {template:"
        t += "            #!/bin/bash -x\n"
        t += "            #Invoke the container\n"
        t += "}\n"
        return t

    def _get_tmpl_for_docker_reg(self, assem, template, image_id):
        template_bdy = yaml.safe_load(template)

        run_docker = ('#!/bin/bash -x\n'
                      '# Invoke the container\n'
                      'docker run -p 80:80 -d {img}\n'
                      'wc_notify --data-binary {stat}')
        run_docker = run_docker.format(img=image_id,
                                       stat='\'{"status": "SUCCESS"}\'')
        comp_instance = template_bdy['resources']['compute_instance']
        user_data = comp_instance['properties']['user_data']
        user_data['str_replace']['template'] = run_docker
        comp_instance['properties']['user_data'] = user_data
        template_bdy['resources']['compute_instance'] = comp_instance
        template = yaml.dump(template_bdy)
        return template

    def _get_tmpl_for_swift(self, assem, template, image_loc, image_name):
        template_bdy = yaml.safe_load(template)

        image_tar_location = image_loc

        run_docker = ('#!/bin/bash -x\n'
                      '# Invoke the container\n'
                      'wget \"{image_tar_location}\" --output-document={du}\n'
                      'docker load < {du}\n'
                      'docker run -p 80:80 -d {du}\n'
                      'wc_notify --data-binary {stat}')
        run_docker = run_docker.format(image_tar_location=image_tar_location,
                                       du=image_name,
                                       stat='\'{"status": "SUCCESS"}\'')

        comp_instance = template_bdy['resources']['compute_instance']
        user_data = comp_instance['properties']['user_data']
        user_data['str_replace']['template'] = run_docker
        comp_instance['properties']['user_data'] = user_data
        template_bdy['resources']['compute_instance'] = comp_instance
        template = yaml.safe_dump(template_bdy,
                                  encoding='utf-8',
                                  allow_unicode=True)
        return template



# Copyright 2014 - Rackspace Hosting.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from mistral.actions import std_actions
from mistral import exceptions
from oslo_log import log as logging

LOG = logging.getLogger('mistral.actions.solum')


class UpdateStackAction(std_actions.HTTPAction):
    def __init__(self, heat_service_url, action_context,
                 stack_name, stack_id, parameters, image_id, template):
        merged_parms = parameters or {}
        merged_parms['image'] = image_id
        super(UpdateStackAction, self).__init__(
            url='%s/stacks/%s/%s' % (heat_service_url, stack_name, stack_id),
            method='PUT',
            headers={'X-Auth-Token': action_context['openstack']['auth_token'],
                     'Content-Type': 'application/json',
                     'User-Agent': 'mistral-heat-plugin',
                     'Accept': 'application/json'},
            body={'stack_id': stack_id,
                  'parameters': merged_parms,
                  'template': template},
            timeout=30)

    def is_sync(self):
        return True


class CheckStackAction(std_actions.HTTPAction):
    def __init__(self, heat_service_url, action_context,
                 stack_name, stack_id):
        super(CheckStackAction, self).__init__(
            url='%s/stacks/%s/%s' % (heat_service_url, stack_name, stack_id),
            method='GET',
            headers={'X-Auth-Token': action_context['openstack']['auth_token'],
                     'Content-Type': 'application/json',
                     'User-Agent': 'mistral-heat-plugin',
                     'Accept': 'application/json'})

    def run(self):
        resp = super(CheckStackAction, self).run()
        LOG.debug('CheckStackAction resp:%s' % resp['content'].get('stack'))
        if resp['content']['stack']['stack_status'] != 'UPDATE_COMPLETE':
            raise exceptions.ActionException(
                'Stack update not complete: stack %(stack_name)s/%(id)s,'
                ' status %(stack_status)s' % resp['content']['stack'])
        return {'stack_status': resp['content']['stack']['stack_status']}

    def test(self):
        return {'stack_status': 'UPDATE_COMPLETE'}

    def is_sync(self):
        return True



# Copyright 2016 - Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import re

from solum.common import exception as exc


def check_url(data):
    # try to use a correct git uri
    pt = re.compile(r'^(http://|https://|git@)(.+)(/|:/)(.+)(.+)(\.git)')
    match = pt.search(data)
    if not match:
        msg = ("Bad git url. Provide git url in the following format: \n"
               "Public repo: https://github.com/<USER>/<REPO>.git\n"
               "Private repo: <http://|git@><HOST>:<USER>/<REPO>.git\n")
        raise exc.BadRequest(reason=msg)



# Copyright 2014 - Rackspace Hosting
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from unittest import mock

from solum.api.controllers.v1 import userlog
from solum import objects
from solum.tests import base
from solum.tests import fakes


@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('solum.api.handlers.userlog_handler.UserlogHandler')
class TestUserlogsController(base.BaseTestCase):
    def setUp(self):
        super(TestUserlogsController, self).setUp()
        objects.load()

    def test_userlogs_get_all(self, UserlogHandler,
                              resp_mock, request_mock):
        hand_get = UserlogHandler.return_value.get_all_by_id
        fake_userlog = fakes.FakeUserlog()
        resource_uuid = fake_userlog.resource_uuid
        hand_get.return_value = [fake_userlog]
        resp = userlog.UserlogsController(resource_uuid).get_all()
        self.assertEqual(fake_userlog.resource_uuid,
                         resp['result'][0].resource_uuid)
        self.assertEqual(fake_userlog.resource_type,
                         resp['result'][0].resource_type)
        self.assertEqual(fake_userlog.location, resp['result'][0].location)
        self.assertEqual(fake_userlog.strategy, resp['result'][0].strategy)
        self.assertEqual(fake_userlog.strategy_info,
                         resp['result'][0].strategy_info)
        hand_get.assert_called_with(resource_uuid)
        self.assertEqual(200, resp_mock.status)
        self.assertIsNotNone(resp)



# Copyright 2014 - Rackspace Hosting
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
from unittest import mock


from solum.api.handlers import userlog_handler
from solum.tests import base
from solum.tests import fakes
from solum.tests import utils


@mock.patch('solum.objects.registry')
class TestUserlogHandler(base.BaseTestCase):
    def setUp(self):
        super(TestUserlogHandler, self).setUp()
        self.ctx = utils.dummy_context()

    def test_userlog_get_all(self, mock_registry):
        mock_registry.UserlogList.get_all.return_value = {}
        handler = userlog_handler.UserlogHandler(self.ctx)
        res = handler.get_all()
        self.assertIsNotNone(res)
        mock_registry.UserlogList.get_all.assert_called_once_with(self.ctx)

    def test_userlog_get_all_by_id(self, mock_registry):
        mock_registry.UserlogList.get_all_by_id.return_value = {}
        handler = userlog_handler.UserlogHandler(self.ctx)
        assembly = fakes.FakeAssembly()
        res = handler.get_all_by_id(assembly.id)
        self.assertIsNotNone(res)
        all_by_id = mock_registry.UserlogList.get_all_by_id
        all_by_id.assert_called_once_with(self.ctx, resource_uuid=assembly.id)

    @mock.patch('solum.api.handlers.userlog_handler.os.remove')
    def test_userlog_delete_local_logs(self, mock_os_remove, mock_registry):
        fi = fakes.FakeImage()
        fakelog = fakes.FakeUserlog()
        flogs = [fakelog]
        mock_registry.UserlogList.get_all_by_id.return_value = flogs

        handler = userlog_handler.UserlogHandler(self.ctx)
        handler.delete(fi.uuid)

        mock_registry.UserlogList.get_all_by_id.assert_called_once_with(
            self.ctx, resource_uuid=fi.uuid)
        mock_os_remove.assert_called_once_with(flogs[0].location)
        fakelog.destroy.assert_called_once_with(self.ctx)

    @mock.patch('solum.common.solum_swiftclient.SwiftClient.delete_object')
    def test_userlog_delete_swift_logs(self, mock_swift_delete, mock_registry):
        fi = fakes.FakeImage()
        fakelog = fakes.FakeUserlog()
        fakelog.strategy = 'swift'
        fakelog.strategy_info = '{"container": "solum-logs"}'
        flogs = [fakelog]
        s_info = json.loads(flogs[0].strategy_info)['container']
        location = flogs[0].location
        mock_registry.UserlogList.get_all_by_id.return_value = flogs

        handler = userlog_handler.UserlogHandler(self.ctx)
        handler.delete(fi.uuid)

        mock_registry.UserlogList.get_all_by_id.assert_called_once_with(
            self.ctx, resource_uuid=fi.uuid)
        mock_swift_delete.assert_called_once_with(s_info, location)
        fakelog.destroy.assert_called_once_with(self.ctx)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from wsme import types as wtypes

from solum.api.controllers import common_types
from solum.api.controllers.v1.datamodel import types as api_types


class Platform(api_types.Base):
    """CAMP v1.1 platform resource."""

    supported_formats_uri = common_types.Uri
    """This attribute is a URL reference to the formats resource for the
     purpose of identifying all supported formats for this platform."""

    extensions_uri = common_types.Uri
    """This attribute is a URL reference to the extensions resource that
     identifies the extensions supported by this platform."""

    type_definitions_uri = common_types.Uri
    """This attribute is a URL reference to the type_definitions resource that
     provides information on the resource types that the Platform supports."""

    platform_endpoints_uri = common_types.Uri
    """This attribute is a URL reference to the platform_endpoints resource."""

    specification_version = wtypes.text
    """The value of this attribute is the Specification Version String of the
     CAMP specification that is supported by the resources rooted in this
     platform."""

    implementation_version = wtypes.text
    """The value of this attribute is a string that expresses the
     provider-specific implementation version supported by the resources
     rooted in this platform."""

    assemblies_uri = common_types.Uri
    """This attribute is a URL reference to the assemblies resource."""

    services_uri = common_types.Uri
    """This attribute is a URL reference to the services resource."""

    plans_uri = common_types.Uri
    """This attribute is a URL reference to the plans resource."""

    def __init__(self, **kwds):
        super(Platform, self).__init__(**kwds)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import rest
import wsmeext.pecan as wsme_pecan

from solum.api.controllers.camp import platform_endpoints as pe
from solum.api.controllers.camp.v1_1.datamodel import platform as model
from solum.api.controllers.camp.v1_1 import uris
from solum.common import exception

uri_string = '%s/camp/v1_1/platform'
description_string = "Solum CAMP API platform resource for CAMP v1.1."


class PlatformController(rest.RestController):
    """CAMP v1.1 platform controller."""

    @exception.wrap_wsme_controller_exception
    @wsme_pecan.wsexpose(model.Platform)
    def get(self):
        host_url = pecan.request.application_url.rstrip('/')
        return model.Platform(uri=uri_string % host_url,
                              name='Solum_CAMP_v1_1_platform',
                              type='platform',
                              description=description_string,
                              supported_formats_uri=uris.FORMATS_URI_STR %
                              host_url,
                              extensions_uri=uris.EXTNS_URI_STR %
                              host_url,
                              type_definitions_uri=uris.TYPE_DEFS_URI_STR %
                              host_url,
                              platform_endpoints_uri=pe.URI_STRING %
                              host_url,
                              specification_version='CAMP 1.1',
                              implementation_version='Solum CAMP 1.1',
                              assemblies_uri=uris.ASSEMS_URI_STR %
                              host_url,
                              services_uri=uris.SERVS_URI_STR %
                              host_url,
                              plans_uri=uris.PLANS_URI_STR %
                              host_url)



# Copyright 2014 - Rackspace Hosting.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os.path

from oslo_config import cfg

from solum.common import exception

OPTS = [
    cfg.StrOpt('source_path',
               default='',
               help='The location the source code.'),
]


def list_opts():
    yield None, OPTS


CONF = cfg.CONF
CONF.register_opts(OPTS)


def get(entity, name, content_type='yaml'):
    """This reads a file's contents from local storage.

    /etc/solum/<entity>/name.<content_type>
    """
    proj_dir = CONF.get('source_path')
    if not proj_dir:
        proj_dir = os.path.join(os.path.dirname(__file__), '..', '..')
    file_path = os.path.join(proj_dir, 'etc', 'solum', entity,
                             '%s.%s' % (name, content_type))
    file_path = os.path.realpath(file_path)
    try:
        with open(file_path) as fd:
            return fd.read()
    except Exception:
        raise exception.ObjectNotFound(
            name=entity, id=name)


def get_from_contrib(name):
    """This reads a file's contents from local storage.

    /contrib/common/name.<content_type>
    """
    proj_dir = CONF.get('source_path')
    if not proj_dir:
        proj_dir = os.path.join(os.path.dirname(__file__), '..', '..')
    file_path = os.path.join(proj_dir, 'contrib', 'common', '%s' % name)
    file_path = os.path.realpath(file_path)
    try:
        with open(file_path) as fd:
            return fd.read()
    except Exception:
        raise exception.ObjectNotFound(id=name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from wsme import types as wtypes

from solum.api.controllers.v1.datamodel import component as component_api
from solum.api.controllers.v1.datamodel import types as api_types
from solum import objects
from solum.objects.sqlalchemy import component as component_model
from solum.tests import base
from solum.tests import utils


class TestTypes(base.BaseTestCase):
    def setUp(self):
        super(TestTypes, self).setUp()
        self.useFixture(utils.Database())

    def test_as_dict(self):
        data = {'name': 'test_hb',
                'uri': 'http://test_host/v1/components/hb',
                'project_id': 'a3266ef8-b3fa-4ab8-b468-d42b5fab5c4d',
                'user_id': '59a9da1f-9a19-4f1e-8877-120865da716b'}
        a = component_api.Component(**data)
        del data['uri']
        self.assertEqual(data, a.as_dict(component_model.Component))

    def test_as_dict_from_keys(self):
        data = {'name': 'test_hb',
                'project_id': 'a3266ef8-b3fa-4ab8-b468-d42b5fab5c4d'}
        a = component_api.Component(**data)
        self.assertEqual(data, a.as_dict_from_keys(['name', 'project_id']))

    def test_from_db_model(self):
        obj = objects.registry.Component()
        data = {'uuid': '8705981d-162b-4431-943e-53609fdc1750',
                'name': 'test_hb',
                'project_id': '3594fb87-487f-44f8-889a-c76bec06174c',
                'user_id': 'd724ff2b-0035-457d-9afc-07f6efa3f0a5',
                'does_not_exist': 'n/a'}
        obj.update(data)
        host_url = 'http://test_host'
        c = component_api.Component.from_db_model(obj, host_url)
        self.assertEqual(data['name'], c.name)
        self.assertEqual(data['project_id'], c.project_id)
        self.assertEqual('component', c.type)
        self.assertEqual(data['user_id'], c.user_id)
        self.assertEqual('%s/v1/%s/%s' % (host_url, obj.__resource__, None),
                         c.uri)
        self.assertIsNone(c.assembly_uuid)

    def test_from_db_model_extra_keys(self):
        assembly_data = {'uuid': '42d',
                         'name': 'test_assembly',
                         'plan_id': 1,
                         'project_id': '3594fb87-487f-44f8-889a-c76bec06174c',
                         'user_id': 'd724ff2b-0035-457d-9afc-07f6efa3f0a5'}
        a = objects.registry.Assembly(**assembly_data)
        a.create(utils.dummy_context())
        data = {'uuid': '8705981d-162b-4431-943e-53609fdc1750',
                'name': 'test_hb',
                'project_id': '3594fb87-487f-44f8-889a-c76bec06174c',
                'user_id': 'd724ff2b-0035-457d-9afc-07f6efa3f0a5',
                'assembly_uuid': assembly_data['uuid']}
        obj = objects.registry.Component(**data)
        c = component_api.Component.from_db_model(obj, 'http://test_host')
        self.assertEqual(assembly_data['uuid'], c.assembly_uuid)


class TestMultiType(base.BaseTestCase):

    def test_valid_values(self):
        vt = api_types.MultiType(wtypes.text, bool)
        value = vt.validate("somestring")
        self.assertEqual("somestring", value)
        value = vt.validate(True)
        self.assertTrue(value)

    def test_invalid_values(self):
        vt = api_types.MultiType(wtypes.text, bool)
        self.assertRaises(ValueError, vt.validate, 10)
        self.assertRaises(ValueError, vt.validate, 0.10)
        self.assertRaises(ValueError, vt.validate, object())

    def test_multitype_tostring(self):
        vt = api_types.MultiType(wtypes.text, bool)
        vts = str(vt)
        self.assertIn(str(wtypes.text), vts)
        self.assertIn(str(bool), vts)
        self.assertNotIn(str(int), vts)


class TestTypeNames(base.BaseTestCase):

    scenarios = [
        ('punct', dict(
            in_value='.,', expect_ok=False)),
        ('-', dict(
            in_value='a-b', expect_ok=True)),
        ('_', dict(
            in_value='a_b', expect_ok=True)),
        ('spaces', dict(
            in_value='look a space', expect_ok=False)),
        ('special', dict(
            in_value='$-&', expect_ok=False)),
        ('upper', dict(
            in_value='ALLGOOD', expect_ok=True)),
        ('lower', dict(
            in_value='evenbetter', expect_ok=True)),
        ('numbers', dict(
            in_value='12345', expect_ok=True)),
    ]

    def test_name(self):
        if self.expect_ok:
            s = component_api.Component(name=self.in_value)
            self.assertEqual(self.in_value, s.name)
        else:
            self.assertRaises(ValueError, component_api.Component,
                              name=self.in_value)



# Copyright 2013 - Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
SQLAlchemy models for application data.
"""

import functools
import json
import time

from oslo_config import cfg
from oslo_db import exception as db_exc
from oslo_db.sqlalchemy import models
from oslo_log import log as logging
from oslo_utils import uuidutils
from sqlalchemy import exc as sqla_exc
from sqlalchemy.ext import declarative
from sqlalchemy.orm import exc as orm_exc
from sqlalchemy import types
from urllib import parse

from solum.common import exception
from solum.common import yamlutils
from solum import objects
from solum.objects import sqlalchemy as object_sqla

LOG = logging.getLogger(__name__)


def retry(fun):
    """Decorator to retry a DB call if certain exception was received."""
    @functools.wraps(fun)
    def _wrapper(*args, **kwargs):
        max_retries = kwargs.pop('max_retries', 3)
        for tries in range(max_retries):
            try:
                return fun(*args, **kwargs)
            except (db_exc.DBDeadlock, orm_exc.StaleDataError,
                    exception.ResourceExists):
                LOG.warning("Failed DB call %s. Retrying %s more times." %
                            (fun.__name__, max_retries - tries - 1))
                if tries + 1 >= max_retries:
                    raise

                time.sleep(0.5)
    return _wrapper


def table_args():
    cfg.CONF.import_opt('connection', 'oslo_db.options',
                        group='database')
    if cfg.CONF.database.connection is None:
        # this is only within some object tests where
        # the object classes are directly imported.
        return None
    engine_name = parse.urlparse(
        cfg.CONF.database.connection).scheme
    if engine_name == 'mysql':
        return {'mysql_engine': 'InnoDB',
                'mysql_charset': "utf8"}
    return None


def filter_by_project(context, query):
    if context is not None:
        if context.is_admin:
            return query
        try:
            query = query.filter_by(project_id=context.project_id)
        except sqla_exc.InvalidRequestError:
            # No project_id column.
            pass
    return query


def model_query(context, model, *args, **kwargs):
    """Query helper.

    :param context: context to query under
    :param session: if present, the session to use
    """

    session = kwargs.get('session') or object_sqla.get_session()

    query = session.query(model, *args)
    return filter_by_project(context, query)


class SolumBase(models.TimestampMixin, models.ModelBase):

    metadata = None

    @classmethod
    def obj_name(cls):
        return cls.__name__

    def as_dict(self):
        d = {}
        for c in self.__table__.columns:
            d[c.name] = self[c.name]
        for k in self._extra_keys:
            d[k] = self[k]
        return d

    @classmethod
    def get_session(cls):
        return object_sqla.get_session()

    @classmethod
    def get_by_id(cls, context, item_id):
        try:
            session = SolumBase.get_session()
            result = session.query(cls).filter_by(id=item_id)
            return filter_by_project(context, result).one()
        except orm_exc.NoResultFound:
            cls._raise_not_found(item_id)

    @classmethod
    def get_by_uuid(cls, context, item_uuid):
        try:
            session = SolumBase.get_session()
            result = session.query(cls).filter_by(uuid=item_uuid)
            return filter_by_project(context, result).one()
        except orm_exc.NoResultFound:
            cls._raise_not_found(item_uuid)

    @classmethod
    def _raise_duplicate_object(cls):
        if hasattr(cls, '__resource__'):
            raise exception.ResourceExists(name=cls.__resource__)
        else:
            raise exception.ObjectNotUnique(name=cls.__tablename__)

    def _non_updatable_fields(self):
        return set(('uuid', 'id'))

    def _lazyhasattr(self, name):
        return any(name in d for d in (self.__dict__,
                                       self.__class__.__dict__))

    def _is_updatable(self):
        return True

    def update(self, data):
        for field in set(data) - self._non_updatable_fields():
            if self._lazyhasattr(field):
                setattr(self, field, data[field])

    @classmethod  # Must be top most
    @retry
    def update_and_save(cls, context, id_or_uuid, data):
        is_uuid = uuidutils.is_uuid_like(id_or_uuid)
        try:
            session = SolumBase.get_session()
            with session.begin():
                if is_uuid:
                    query = session.query(cls).filter_by(uuid=id_or_uuid)
                else:
                    query = session.query(cls).filter_by(id=id_or_uuid)
                obj = filter_by_project(context, query).one()
                if obj._is_updatable():
                    obj.update(data)
                    session.merge(obj)
            return obj
        except orm_exc.NoResultFound:
            cls._raise_not_found(id_or_uuid)

    @retry
    def save(self, context):
        if objects.transition_schema():
            self.add_forward_schema_changes()

        session = SolumBase.get_session()
        with session.begin():
            session.merge(self)

    @retry
    def create(self, context):
        session = SolumBase.get_session()
        try:
            with session.begin():
                session.add(self)
        except (db_exc.DBDuplicateEntry):
            self.__class__._raise_duplicate_object()

    @retry
    def destroy(self, context):
        session = SolumBase.get_session()
        with session.begin():
            session.query(self.__class__).filter_by(
                id=self.id).delete()

    @classmethod
    def _raise_not_found(cls, item_id):
        """Raise a not found exception."""
        if hasattr(cls, '__resource__'):
            raise exception.ResourceNotFound(name=cls.__resource__, id=item_id)
        else:
            raise exception.ObjectNotFound(name=cls.__tablename__, id=item_id)


Base = declarative.declarative_base(cls=SolumBase)


class JSONEncodedDict(types.TypeDecorator):
    """Represents an immutable structure as a json-encoded string."""

    impl = types.TEXT

    def process_bind_param(self, value, dialect):
        if value is not None:
            value = json.dumps(value)
        return value

    def process_result_value(self, value, dialect):
        if value is not None:
            value = json.loads(value)
        return value


class YAMLEncodedDict(types.TypeDecorator):
    """Represents an immutable structure as a yaml-encoded string."""

    impl = types.TEXT

    def process_bind_param(self, value, dialect):
        if value is not None:
            value = yamlutils.dump(value)
        return value

    def process_result_value(self, value, dialect):
        if value is not None:
            value = yamlutils.load(value)
        return value



# Copyright 2013 - Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
from unittest import mock


from solum.api.controllers.v1.datamodel import service as servicemodel
from solum.api.controllers.v1 import service
from solum.common import exception
from solum import objects
from solum.tests import base
from solum.tests import fakes


@mock.patch('solum.common.policy.check')
@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('solum.api.handlers.service_handler.ServiceHandler')
class TestServiceController(base.BaseTestCase):
    def setUp(self):
        super(TestServiceController, self).setUp()
        objects.load()

    def test_service_get(self, ServiceHandler, resp_mock,
                         request_mock, mock_policy):
        mock_policy.return_value = True
        hand_get = ServiceHandler.return_value.get
        fake_service = fakes.FakeService()
        hand_get.return_value = fake_service
        cont = service.ServiceController('test_id')
        resp = cont.get()
        self.assertIsNotNone(resp)
        self.assertEqual(fake_service.name, resp['result'].name)
        self.assertEqual(fake_service.description,
                         resp['result'].description)
        self.assertEqual(fake_service.project_id, resp['result'].project_id)
        self.assertEqual(fake_service.uuid, resp['result'].uuid)
        hand_get.assert_called_with('test_id')
        self.assertEqual(200, resp_mock.status)

    def test_service_get_not_found(self, ServiceHandler,
                                   resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        hand_get = ServiceHandler.return_value.get
        hand_get.side_effect = exception.ResourceNotFound(
            name='service', service_id='test_id')
        cont = service.ServiceController('test_id')
        cont.get()
        hand_get.assert_called_with('test_id')
        self.assertEqual(404, resp_mock.status)

    def test_service_put_none(self, ServiceHandler, resp_mock,
                              request_mock, mock_policy):
        mock_policy.return_value = True
        request_mock.body = None
        request_mock.content_type = 'application/json'
        hand_put = ServiceHandler.return_value.put
        hand_put.return_value = fakes.FakeService()
        service.ServiceController('test_id').put()
        self.assertEqual(400, resp_mock.status)

    def test_service_put_not_found(self, ServiceHandler,
                                   resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        json_update = {'user_id': 'foo', 'name': 'appy'}
        request_mock.body = json.dumps(json_update)
        request_mock.content_type = 'application/json'
        hand_update = ServiceHandler.return_value.update
        hand_update.side_effect = exception.ResourceNotFound(
            name='service',
            service_id='test_id')
        service.ServiceController('test_id').put()
        hand_update.assert_called_with('test_id', json_update)
        self.assertEqual(404, resp_mock.status)

    def test_service_put_ok(self, ServiceHandler, resp_mock,
                            request_mock, mock_policy):
        mock_policy.return_value = True
        json_update = {'name': 'fee', 'user_id': 'me'}
        request_mock.body = json.dumps(json_update)
        request_mock.content_type = 'application/json'
        hand_update = ServiceHandler.return_value.update
        hand_update.return_value = fakes.FakeService()
        service.ServiceController('test_id').put()
        hand_update.assert_called_with('test_id', json_update)
        self.assertEqual(200, resp_mock.status)

    def test_service_delete_not_found(self, ServiceHandler,
                                      resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        hand_delete = ServiceHandler.return_value.delete
        hand_delete.side_effect = exception.ResourceNotFound(
            name='service',
            service_id='test_id')
        obj = service.ServiceController('test_id')
        obj.delete()
        hand_delete.assert_called_with('test_id')
        self.assertEqual(404, resp_mock.status)

    def test_service_delete_ok(self, ServiceHandler, resp_mock,
                               request_mock, mock_policy):
        mock_policy.return_value = True
        hand_delete = ServiceHandler.return_value.delete
        hand_delete.return_value = None
        obj = service.ServiceController('test_id')
        obj.delete()
        hand_delete.assert_called_with('test_id')
        self.assertEqual(204, resp_mock.status)


@mock.patch('solum.common.policy.check')
@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('solum.api.handlers.service_handler.ServiceHandler')
class TestServicesController(base.BaseTestCase):
    def setUp(self):
        super(TestServicesController, self).setUp()
        objects.load()

    def test_services_get_all(self, handler_mock, resp_mock,
                              request_mock, mock_policy):
        mock_policy.return_value = True
        hand_get_all = handler_mock.return_value.get_all
        fake_service = fakes.FakeService()
        hand_get_all.return_value = [fake_service]
        obj = service.ServicesController()
        resp = obj.get_all()
        self.assertIsNotNone(resp)
        self.assertEqual(fake_service.name, resp['result'][0].name)
        self.assertEqual(fake_service.description,
                         resp['result'][0].description)
        self.assertEqual(fake_service.project_id, resp['result'][0].project_id)
        self.assertEqual(fake_service.uuid, resp['result'][0].uuid)
        hand_get_all.assert_called_with()
        self.assertEqual(200, resp_mock.status)

    def test_services_post(self, handler_mock, resp_mock,
                           request_mock, mock_policy):
        mock_policy.return_value = True
        json_create = {'name': 'foo',
                       'description': 'test_desc_service',
                       'user_id': 'user_id_test',
                       'project_id': 'project_id_test'}
        request_mock.body = json.dumps(json_create)
        request_mock.content_type = 'application/json'
        handler_create = handler_mock.return_value.create
        handler_create.return_value = fakes.FakeService()
        service.ServicesController().post()
        handler_create.assert_called_with(json_create)
        self.assertEqual(201, resp_mock.status)
        handler_create.assert_called_once_with(json_create)

    def test_services_post_nodata(self, handler_mock, resp_mock,
                                  request_mock, mock_policy):
        mock_policy.return_value = True
        request_mock.body = ''
        request_mock.content_type = 'application/json'
        handler_create = handler_mock.return_value.create
        handler_create.return_value = fakes.FakeService()
        ret_val = service.ServicesController().post()
        self.assertEqual("Missing argument: \"data\"",
                         str(ret_val['faultstring']))
        self.assertEqual(400, resp_mock.status)


class TestServiceAsDict(base.BaseTestCase):

    scenarios = [
        ('none', dict(data=None)),
        ('one', dict(data={'name': 'foo'})),
        ('full', dict(data={'uri': 'http://example.com/v1/services/x1',
                            'name': 'Example-service',
                            'type': 'service',
                            'tags': ['small'],
                            'project_id': '1dae5a09ef2b4d8cbf3594b0eb4f6b94',
                            'user_id': '55f41cf46df74320b9486a35f5d28a11',
                            'description': 'A service'}))
    ]

    def test_as_dict(self):
        objects.load()
        if self.data is None:
            s = servicemodel.Service()
            self.data = {}
        else:
            s = servicemodel.Service(**self.data)
        if 'uri' in self.data:
            del self.data['uri']
        if 'type' in self.data:
            del self.data['type']
        self.assertEqual(self.data,
                         s.as_dict(objects.registry.Service))



# Copyright 2014 - Rackspace Hosting
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""Solum Deployer noop handler."""

from oslo_log import log as logging

from solum import objects

LOG = logging.getLogger(__name__)


class Handler(object):
    def echo(self, ctxt, message):
        LOG.debug("%s" % message)

    def deploy(self, ctxt, assembly_id, image_id, ports):
        message = ("Deploy %s %s %s" % (assembly_id, image_id, ports))
        LOG.debug("%s" % message)

    def destroy_assembly(self, ctxt, assem_id):
        assem = objects.registry.Assembly.get_by_id(ctxt, assem_id)
        assem.destroy(ctxt)
        message = ("Destroy %s" % (assem_id))
        LOG.debug("%s" % message)

    def destroy_app(self, ctxt, app_id):
        message = ("Destroy App %s" % (app_id))
        LOG.debug("%s" % message)



# Copyright 2013 - Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Solum base exception handling.

Includes decorator for re-raising Solum-type exceptions.

"""

from collections import abc
import functools
import inspect
import sys

from keystoneclient import exceptions as keystone_exceptions
from oslo_config import cfg
from oslo_log import log as logging
from oslo_utils import excutils
from oslo_utils import uuidutils
import pecan
import wsme

from solum.i18n import _


LOG = logging.getLogger(__name__)

exc_log_opts = [
    cfg.BoolOpt('fatal_exception_format_errors',
                default=False,
                help='make exception message format errors fatal')
]


def list_opts():
    yield None, exc_log_opts


CONF = cfg.CONF
CONF.register_opts(exc_log_opts)


def wrap_exception(notifier=None, publisher_id=None, event_type=None,
                   level=None):
    """This decorator wraps a method to catch any exceptions.

    It logs the exception as well as optionally sending
    it to the notification system.
    """
    def inner(f):
        def wrapped(self, context, *args, **kw):
            # Don't store self or context in the payload, it now seems to
            # contain confidential information.
            try:
                return f(self, context, *args, **kw)
            except Exception as e:
                with excutils.save_and_reraise_exception():
                    if notifier:
                        call_dict = inspect.getcallargs(f, *args, **kw)
                        payload = dict(exception=e,
                                       private=dict(args=call_dict)
                                       )

                        # Use a temp vars so we don't shadow
                        # our outer definitions.
                        temp_level = level
                        if not temp_level:
                            temp_level = notifier.ERROR

                        temp_type = event_type
                        if not temp_type:
                            # If f has multiple decorators, they must use
                            # functools.wraps to ensure the name is
                            # propagated.
                            temp_type = f.__name__

                        notifier.notify(context, publisher_id, temp_type,
                                        temp_level, payload)

        return functools.wraps(f)(wrapped)
    return inner


OBFUSCATED_MSG = _('Your request could not be handled '
                   'because of a problem in the server. '
                   'Error Correlation id is: %s')


def wrap_controller_exception(func, func_server_error, func_client_error):
    """This decorator wraps controllers methods to handle exceptions:

    - if an unhandled Exception or a SolumException with an error code >=500
    is catched, raise a http 5xx ClientSideError and correlates it with a log
    message

    - if a SolumException is catched and its error code is <500, raise a http
    4xx and logs the exception in debug mode

    """
    @functools.wraps(func)
    def wrapped(*args, **kw):
        try:
            return func(*args, **kw)
        except Exception as excp:
            LOG.error(excp)
            http_error_code = 500
            if hasattr(excp, 'code'):
                http_error_code = excp.code

            if http_error_code >= 500:
                # log the error message with its associated
                # correlation id
                log_correlation_id = uuidutils.generate_uuid()
                LOG.error("%s:%s", log_correlation_id, str(excp))
                # raise a client error with an obfuscated message
                func_server_error(log_correlation_id, http_error_code)
            else:
                # raise a client error the original message
                func_client_error(excp, http_error_code)
    return wrapped


def wrap_wsme_controller_exception(func):
    """This decorator wraps wsme controllers to handle exceptions."""
    def _func_server_error(log_correlation_id, status_code):
        raise wsme.exc.ClientSideError(
            str(OBFUSCATED_MSG % log_correlation_id), status_code)

    def _func_client_error(excp, status_code):
        raise wsme.exc.ClientSideError(str(excp), status_code)

    return wrap_controller_exception(func,
                                     _func_server_error,
                                     _func_client_error)


def wrap_pecan_controller_exception(func):
    """This decorator wraps pecan controllers to handle exceptions."""
    def _func_server_error(log_correlation_id, status_code):
        pecan.response.status = status_code
        pecan.response.text = str(OBFUSCATED_MSG % log_correlation_id)
        # message body for errors is just a plain text message
        # The following code is functionally equivalent to calling:
        #
        #     pecan.override_template(None, "text/plain")
        #
        # We do it this way to work around a bug in our unit-test framework
        # in which the mocked request object isn't properly mocked in the pecan
        # core module (gilbert.plz@oracle.com)
        pecan.request.pecan['override_template'] = None
        pecan.request.pecan['override_content_type'] = 'text/plain'

    def _func_client_error(excp, status_code):
        pecan.response.status = status_code
        pecan.response.text = str(excp)

        # The following code is functionally equivalent to calling:
        #
        #     pecan.override_template(None, "text/plain")
        #
        # We do it this way to work around a bug in our unit-test framework
        # in which the mocked request object isn't properly mocked in the pecan
        # core module (gilbert.plz@oracle.com)
        pecan.request.pecan['override_template'] = None
        pecan.request.pecan['override_content_type'] = 'text/plain'

    return wrap_controller_exception(func,
                                     _func_server_error,
                                     _func_client_error)


def wrap_wsme_pecan_controller_exception(func):
    """Error handling for controllers decorated with wsmeext.pecan.wsexpose:

    Controllers wrapped with wsme_pecan.wsexpose don't throw
    exceptions but handle them internally. We need to intercept
    the response and mask potentially sensitive information.
    """

    @functools.wraps(func)
    def wrapped(*args, **kw):
        ret = func(*args, **kw)
        ismapping = isinstance(ret, abc.Mapping)
        if (pecan.response.status_code >= 500 and ismapping):

            log_correlation_id = uuidutils.generate_uuid()
            LOG.error("%s:%s", log_correlation_id, ret.get("faultstring",
                                                           "Unknown Error"))
            ret['faultstring'] = str(OBFUSCATED_MSG % log_correlation_id)
        return ret

    return wrapped


def wrap_keystone_exception(func):
    """Wrap keystone exceptions and throw Solum specific exceptions."""
    @functools.wraps(func)
    def wrapped(*args, **kw):
        try:
            return func(*args, **kw)
        except keystone_exceptions.AuthorizationFailure:
            raise AuthorizationFailure(
                client=func.__name__, message="reason: %s" % sys.exc_info()[1])
        except keystone_exceptions.ClientException:
            raise AuthorizationFailure(
                client=func.__name__,
                message="unexpected keystone client error occurred: %s"
                        % sys.exc_info()[1])
    return wrapped


class SolumException(Exception):
    """Base Solum Exception

    To correctly use this class, inherit from it and define
    a 'msg_fmt' property. That msg_fmt will get printf'd
    with the keyword arguments provided to the constructor.

    """
    message = _("An unknown exception occurred.")
    code = 500

    def __init__(self, **kwargs):
        self.kwargs = kwargs

        if CONF.fatal_exception_format_errors:
            assert isinstance(self.msg_fmt, str)

        try:
            self.message = self.msg_fmt % kwargs
        except KeyError:
            # kwargs doesn't match a variable in the message
            # log the issue and the kwargs
            LOG.exception(_('Exception in string format operation'),
                          extra=dict(
                              private=dict(
                                  msg=self.msg_fmt,
                                  args=kwargs
                                  )
                              )
                          )

            if CONF.fatal_exception_format_errors:
                raise

    def __str__(self):
        return self.message


class ResourceLimitExceeded(SolumException):
    msg_fmt = _("Resource limit exceeded. Reason: %(reason)s")


class BadRequest(SolumException):
    msg_fmt = _("The request is malformed. Reason: %(reason)s")
    code = 400


class ObjectNotFound(SolumException):
    msg_fmt = _("The %(name)s %(id)s could not be found.")


class ObjectNotUnique(SolumException):
    msg_fmt = _("The %(name)s already exists.")


class RequestForbidden(SolumException):
    msg_fmt = _("The request is forbidden. Reason: %(reason)s")
    code = 403


class ResourceNotFound(ObjectNotFound):
    msg_fmt = _("The %(name)s resource %(id)s could not be found.")
    code = 404


class ResourceExists(ObjectNotUnique):
    msg_fmt = _("The %(name)s resource already exists.")
    code = 409


class ResourceStillReferenced(SolumException):
    msg_fmt = _("The %(name)s resource cannot be deleted because one or more"
                " resources reference it.")
    code = 409


class UnsupportedMediaType(SolumException):
    msg_fmt = _("\'%(name)s\' is not a supported media type for the %(method)s"
                " method of this resource")
    code = 415


class Unprocessable(SolumException):
    msg_fmt = _("Server is incapable of processing the specified request.")
    code = 422


class PlanStillReferenced(ResourceStillReferenced):
    msg_fmt = _("Plan %(name)s cannot be deleted because one or more"
                " Assemblies reference it.")


class LPStillReferenced(ResourceStillReferenced):
    msg_fmt = _("Languagepack %(name)s cannot be deleted because one or more"
                " applications reference it.")


class NotImplemented(SolumException):
    msg_fmt = _("The requested operation is not implemented.")
    code = 501


class AuthorizationFailure(SolumException):
    msg_fmt = _("%(client)s connection failed. %(message)s")


class InvalidObjectSizeError(Exception):
    msg_fmt = _("Invalid object size.")


class MaxRetryReached(Exception):
    msg_fmt = _("Maximum retries has been reached.")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import rest
from wsme import types as wtypes
import wsmeext.pecan as wsme_pecan

from solum.api.controllers.v1.datamodel import operation
from solum.api.handlers import operation_handler
from solum.common import exception
from solum.common import policy
from solum import objects


class OperationController(rest.RestController):
    """Manages operations on a single operation."""

    def __init__(self, operation_id):
        super(OperationController, self).__init__()
        self._id = operation_id

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose(operation.Operation, wtypes.text)
    def get(self):
        """Return this operation."""
        policy.check('show_operation',
                     pecan.request.security_context)
        handler = operation_handler.OperationHandler(
            pecan.request.security_context)
        host_url = pecan.request.application_url.rstrip('/')
        return operation.Operation.from_db_model(handler.get(self._id),
                                                 host_url)

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose(operation.Operation, wtypes.text,
                         body=operation.Operation)
    def put(self, data):
        """Modify this operation."""
        policy.check('update_operation',
                     pecan.request.security_context)
        handler = operation_handler.OperationHandler(
            pecan.request.security_context)
        res = handler.update(self._id,
                             data.as_dict(objects.registry.Operation))
        host_url = pecan.request.application_url.rstrip('/')
        return operation.Operation.from_db_model(res, host_url)

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose(status_code=204)
    def delete(self):
        """Delete this operation."""
        policy.check('delete_operation',
                     pecan.request.security_context)
        handler = operation_handler.OperationHandler(
            pecan.request.security_context)
        handler.delete(self._id)


class OperationsController(rest.RestController):
    """Manages operations on the operations collection."""

    @pecan.expose()
    def _lookup(self, operation_id, *remainder):
        if remainder and not remainder[-1]:
            remainder = remainder[:-1]
        return OperationController(operation_id), remainder

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose(operation.Operation, body=operation.Operation,
                         status_code=201)
    def post(self, data):
        """Create a new operation."""
        policy.check('create_operation',
                     pecan.request.security_context)
        handler = operation_handler.OperationHandler(
            pecan.request.security_context)
        host_url = pecan.request.application_url.rstrip('/')
        return operation.Operation.from_db_model(handler.create(
            data.as_dict(objects.registry.Operation)), host_url)

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose([operation.Operation])
    def get_all(self):
        """Return all operations, based on the query provided."""
        policy.check('get_operations',
                     pecan.request.security_context)
        handler = operation_handler.OperationHandler(
            pecan.request.security_context)
        host_url = pecan.request.application_url.rstrip('/')
        return [operation.Operation.from_db_model(obj, host_url)
                for obj in handler.get_all()]



# Copyright 2013 - Rackspace US, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
import re
import sys

from oslo_db import exception as db_exc
from oslo_log import log as logging
import pecan
from pecan import rest
from wsme.rest import json as wsme_json
from wsme import types as wsme_types
import wsmeext.pecan as wsme_pecan

from solum.api.controllers.v1.datamodel import plan
from solum.api.handlers import plan_handler
from solum.common import exception
from solum.common import policy
from solum.common import yamlutils
from solum import objects

LOG = logging.getLogger(__name__)


def init_plan_v1(yml_input_plan):
    if not yml_input_plan.get('name'):
        raise exception.BadRequest(reason="Name field is missing.")
    try:
        pp = plan.Plan(**yml_input_plan)
    except ValueError as ve:
        raise exception.BadRequest(reason=str(ve))
    try:
        name_regex = re.compile(r'^([a-zA-Z0-9-_]{1,100})$')
        assert name_regex.match(pp.name), 'Plan name is invalid.'
    except AssertionError as ae:
        raise exception.BadRequest(reason=str(ae))
    return pp


def init_plan_by_version(input_plan):
    version = input_plan.get('version')
    if version is None:
        raise exception.BadRequest(
            reason='Version attribute is missing in Plan')
    mod = sys.modules[__name__]
    if not hasattr(mod, 'init_plan_v%s' % version):
        raise exception.BadRequest(reason='Plan version %s is invalid.'
                                          % version)
    return getattr(mod, 'init_plan_v%s' % version)(input_plan)


def init_yml_plan_by_version():
    try:
        yml_input_plan = yamlutils.load(pecan.request.body)
    except ValueError as excp:
        LOG.error("Invalid plan.")
        raise exception.BadRequest(reason='Plan is invalid. '
                                          + str(excp))
    return init_plan_by_version(yml_input_plan)


def init_json_plan_by_version():
    try:
        json_input_plan = json.loads(pecan.request.body)
    except ValueError as excp:
        raise exception.BadRequest(reason='Plan is invalid. '
                                          + str(excp))
    return init_plan_by_version(json_input_plan)


def yaml_content(m):
    ref_content = m.refined_content()
    host_url = pecan.request.application_url.rstrip('/')
    ref_content['uri'] = '%s/v1/plans/%s' % (host_url, m.uuid)
    ref_content['trigger_uri'] = ('%s/v1/triggers/%s' %
                                  (host_url, m.trigger_id))
    return ref_content


class PlanController(rest.RestController):
    """Manages operations on a single plan."""

    def __init__(self, plan_id):
        super(PlanController, self).__init__()
        self._id = plan_id

    @exception.wrap_pecan_controller_exception
    @pecan.expose()
    def get(self):
        """Return this plan."""
        policy.check('show_plan',
                     pecan.request.security_context)
        handler = plan_handler.PlanHandler(pecan.request.security_context)

        host_url = pecan.request.application_url.rstrip('/')
        if pecan.request.accept is not None and 'yaml' in pecan.request.accept:
            plan_serialized = yamlutils.dump(
                yaml_content(handler.get(self._id)))
        else:
            plan_model = plan.Plan.from_db_model(
                handler.get(self._id), host_url)
            plan_serialized = wsme_json.encode_result(plan_model, plan.Plan)
        pecan.response.status = 200
        return plan_serialized

    @exception.wrap_pecan_controller_exception
    @pecan.expose()
    def put(self):
        """Modify this plan."""
        policy.check('update_plan',
                     pecan.request.security_context)
        # make sure the plan exists before parsing the request
        handler = plan_handler.PlanHandler(pecan.request.security_context)
        handler.get(self._id)

        host_url = pecan.request.application_url.rstrip('/')
        if not pecan.request.body or len(pecan.request.body) < 1:
            raise exception.BadRequest(reason="No data.")

        if (pecan.request.content_type is not None and
                'yaml' in pecan.request.content_type):
            data = init_yml_plan_by_version()
            updated_plan_yml = yamlutils.dump(yaml_content(handler.update(
                self._id, data.as_dict(objects.registry.Plan))))
        else:
            data = init_json_plan_by_version()
            plan_obj = handler.update(self._id,
                                      data.as_dict(objects.registry.Plan))
            updated_plan_yml = wsme_json.encode_result(plan.Plan.from_db_model(
                plan_obj, host_url), plan.Plan)

        pecan.response.status = 200
        return updated_plan_yml

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose(status_code=202)
    def delete(self):
        """Delete this plan."""
        policy.check('delete_plan',
                     pecan.request.security_context)
        p_handler = plan_handler.PlanHandler(pecan.request.security_context)
        try:
            p_handler.delete(self._id)
        except (db_exc.DBError):
            raise exception.PlanStillReferenced(name=self._id)


class PlansController(rest.RestController):
    """Manages operations on the plans collection."""

    @pecan.expose()
    def _lookup(self, plan_id, *remainder):
        if remainder and not remainder[-1]:
            remainder = remainder[:-1]
        return PlanController(plan_id), remainder

    @exception.wrap_pecan_controller_exception
    @pecan.expose()
    def post(self):
        """Create a new plan."""
        policy.check('create_plan',
                     pecan.request.security_context)
        if not pecan.request.body or len(pecan.request.body) < 1:
            raise exception.BadRequest(reason="No data.")

        handler = plan_handler.PlanHandler(pecan.request.security_context)

        host_url = pecan.request.application_url.rstrip('/')
        if (pecan.request.content_type is not None and
                'yaml' in pecan.request.content_type):
            data = init_yml_plan_by_version()
            created_plan = yamlutils.dump(yaml_content(handler.create(
                data.as_dict(objects.registry.Plan))))
        else:
            data = init_json_plan_by_version()
            plan_wsme = plan.Plan.from_db_model(handler.create(
                data.as_dict(objects.registry.Plan)), host_url)
            created_plan = wsme_json.encode_result(plan_wsme, plan.Plan)

        pecan.response.status = 201
        return created_plan

    @exception.wrap_pecan_controller_exception
    @pecan.expose()
    def get_all(self):
        """Return all plans, based on the query provided."""
        policy.check('get_plans',
                     pecan.request.security_context)
        handler = plan_handler.PlanHandler(pecan.request.security_context)

        host_url = pecan.request.application_url.rstrip('/')
        if pecan.request.accept is not None and 'yaml' in pecan.request.accept:
            plan_serialized = yamlutils.dump([yaml_content(obj)
                                              for obj in handler.get_all()
                                              if obj and obj.raw_content])
        else:
            plan_serialized = wsme_json.encode_result(
                [plan.Plan.from_db_model(obj, host_url)
                 for obj in handler.get_all()],
                wsme_types.ArrayType(plan.Plan))
        pecan.response.status = 200
        return plan_serialized



# Copyright 2014 - Rackspace Hosting
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from solum.common.rpc import service
from solum.tests import base


class ServiceTest(base.BaseTestCase):

    def test_create(self):
        topic, server, handlers = 'fake_topic', 'fake_host', []
        rpc_service = service.Service(topic, server, handlers)
        self.assertIsNotNone(rpc_service._server)


class APITest(base.BaseTestCase):

    def test_create(self):
        topic = 'fake_topic'
        rpc_api = service.API(context={}, topic=topic)
        self.assertIsNotNone(rpc_api._client)

    def test_cast(self):
        topic = 'fake_topic'
        rpc_api = service.API(context={}, topic=topic)
        rpc_api._cast = mock.MagicMock()
        rpc_api.echo('foo')
        rpc_api._cast.assert_called_once_with('echo', message='foo')



# Copyright 2014 - Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from solum.objects import base


class Assembly(base.CrudMixin):
    # Version 1.0: Initial version
    VERSION = '1.0'


class AssemblyList(list, base.CrudListMixin):
    """List of Assemblies."""


class States(object):
    QUEUED = 'QUEUED'
    UNIT_TESTING = 'UNIT_TESTING'
    UNIT_TESTING_FAILED = 'UNIT_TESTING_FAILED'
    UNIT_TESTING_PASSED = 'UNIT_TESTING_PASSED'
    BUILDING = 'BUILDING'
    BUILT = 'BUILT'
    DEPLOYING = 'DEPLOYING'
    ERROR = 'ERROR'
    DEPLOYMENT_COMPLETE = 'DEPLOYMENT_COMPLETE'
    DELETING = 'DELETING'
    ERROR_STACK_DELETE_FAILED = 'ERROR_STACK_DELETE_FAILED'
    ERROR_STACK_CREATE_FAILED = 'ERROR_STACK_CREATE_FAILED'
    ERROR_CODE_DEPLOYMENT = 'ERROR_CODE_DEPLOYMENT'
    STARTING_APP = 'STARTING_APP'



# Copyright 2014 - Rackspace Hosting
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""Solum Worker noop handler."""

from oslo_log import log as logging

LOG = logging.getLogger(__name__)


class Handler(object):
    def echo(self, ctxt, message):
        LOG.debug("%s" % message)

    def build(self, ctxt, build_id, source_uri, name, base_image_id,
              source_format, image_format, assembly_id,
              test_cmd, artifact_type=None):
        args = [build_id, source_uri, name, base_image_id, source_format,
                image_format, assembly_id, test_cmd, artifact_type]
        message = 'Build ' + ', '.join([str(a) for a in args])
        LOG.debug("%s" % message)

    def unittest(self, ctxt, build_id, source_uri, name, base_image_id,
                 source_format, image_format, assembly_id,
                 test_cmd):
        args = [build_id, source_uri, name, base_image_id, source_format,
                image_format, assembly_id, test_cmd]
        message = 'Unittest ' + ', '.join([str(a) for a in args])
        LOG.debug("%s" % message)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import rest
from wsme import types as wtypes
import wsmeext.pecan as wsme_pecan

from solum.api.controllers.v1.datamodel import sensor
from solum.api.handlers import sensor_handler
from solum.common import exception
from solum.common import policy
from solum import objects


class SensorController(rest.RestController):
    """Manages operations on a single sensor."""

    def __init__(self, sensor_id):
        super(SensorController, self).__init__()
        self._id = sensor_id

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose(sensor.Sensor, wtypes.text)
    def get(self):
        """Return this sensor."""
        policy.check('show_sensor',
                     pecan.request.security_context)
        handler = sensor_handler.SensorHandler(pecan.request.security_context)
        host_url = pecan.request.application_url.rstrip('/')
        return sensor.Sensor.from_db_model(handler.get(self._id), host_url)

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose(sensor.Sensor, wtypes.text, body=sensor.Sensor)
    def put(self, data):
        """Modify this sensor."""
        policy.check('update_sensor',
                     pecan.request.security_context)
        handler = sensor_handler.SensorHandler(pecan.request.security_context)
        obj = handler.update(self._id,
                             data.as_dict(objects.registry.Sensor))
        host_url = pecan.request.application_url.rstrip('/')
        return sensor.Sensor.from_db_model(obj, host_url)

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose(None, wtypes.text, status_code=204)
    def delete(self):
        """Delete this sensor."""
        policy.check('delete_sensor',
                     pecan.request.security_context)
        handler = sensor_handler.SensorHandler(pecan.request.security_context)
        handler.delete(self._id)


class SensorsController(rest.RestController):
    """Manages operations on the sensors collection."""

    @pecan.expose()
    def _lookup(self, sensor_id, *remainder):
        if remainder and not remainder[-1]:
            remainder = remainder[:-1]
        return SensorController(sensor_id), remainder

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose(sensor.Sensor, wtypes.text,
                         body=sensor.Sensor, status_code=201)
    def post(self, data):
        """Create a new sensor."""
        policy.check('create_sensor',
                     pecan.request.security_context)
        handler = sensor_handler.SensorHandler(pecan.request.security_context)
        obj = handler.create(data.as_dict(objects.registry.Sensor))
        host_url = pecan.request.application_url.rstrip('/')
        return sensor.Sensor.from_db_model(obj, host_url)

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose([sensor.Sensor])
    def get_all(self):
        """Return all sensors, based on the query provided."""
        policy.check('get_sensors',
                     pecan.request.security_context)
        handler = sensor_handler.SensorHandler(pecan.request.security_context)
        host_url = pecan.request.application_url.rstrip('/')
        return [sensor.Sensor.from_db_model(obj, host_url)
                for obj in handler.get_all()]



# Copyright 2014 - Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from barbicanclient import client as barbicanclient
from keystoneauth1 import loading

from oslo_config import cfg


class BarbicanClient(object):
    """Barbican client wrapper so we can encapsulate logic in one place."""

    def __init__(self, verify=True):
        self._admin_client = None

    @property
    def admin_client(self):
        if not self._admin_client:
            # Create connection to API
            self._admin_client = self._barbican_admin_init()
        return self._admin_client

    def _barbican_admin_init(self):
        # Import auth_token to have keystone_authtoken settings setup.
        auth = loading.load_auth_from_conf_options(
            cfg.CONF, 'keystone_authtoken')
        sess = loading.load_session_from_conf_options(
            cfg.CONF, 'keystone_authtoken', auth=auth)
        return barbicanclient.Client(session=sess)



# Copyright 2014 - Rackspace Hosting.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from keystoneauth1 import identity
from keystoneauth1 import loading as ka_loading
from keystoneauth1 import session as ks_session
import keystoneclient.exceptions as kc_exception
from keystoneclient.v3 import client as kc_v3
from oslo_config import cfg
from oslo_log import log as logging
from oslo_utils import importutils

from solum.common import context
from solum.common import exception
from solum.i18n import _

LOG = logging.getLogger(__name__)

trust_opts = [
    cfg.ListOpt('trusts_delegated_roles',
                default=['solum_assembly_update'],
                help=_('Subset of trustor roles to be delegated to solum.')),
]
cfg.CONF.register_opts(trust_opts)
cfg.CONF.import_opt('www_authenticate_uri', 'keystonemiddleware.auth_token',
                    group='keystone_authtoken')

AUTH_OPTS = [
    cfg.StrOpt('keystone_version',
               default='3',
               help='The keystone version to use with Solum'),
]


def list_opts():
    yield None, AUTH_OPTS
    yield None, trust_opts


cfg.CONF.register_opts(AUTH_OPTS)

cfg.CONF.import_opt('lp_operator_user', 'solum.worker.config', group='worker')
cfg.CONF.import_opt('lp_operator_password',
                    'solum.worker.config', group='worker')
cfg.CONF.import_opt('lp_operator_tenant_name',
                    'solum.worker.config', group='worker')


class KeystoneClient(object):
    """Keystone client wrapper to initialize the right version of the client.

       The version to use is specified in solum.conf
       """

    def __new__(self, context):
        ks_version = cfg.CONF.get('keystone_version')
        if ks_version == '3':
            return KeystoneClientV3(context)
        else:
            msg = 'Unsupported version of keystone: %s', ks_version
            LOG.error(msg)
            raise exception.AuthorizationFailure(client='keystone',
                                                 message=msg)


class KeystoneClientV3(object):
    """Keystone client wrapper so we can encapsulate logic in one place."""

    def __init__(self, context):
        # If a trust_id is specified in the context, we immediately
        # authenticate so we can populate the context with a trust token
        # otherwise, we delay client authentication until needed to avoid
        # unnecessary calls to keystone.
        #
        # Note that when you obtain a token using a trust, it cannot be
        # used to reauthenticate and get another token, so we have to
        # get a new trust-token even if context.auth_token is set.
        #
        # - context.auth_url is expected to contain a versioned keystone
        #   path, we will work with either a v2.0 or v3 path
        self.context = context
        self._client = None
        self._admin_client = None
        self._lp_admin_client = None

        if self.context and self.context.auth_url:
            self.endpoint = self.context.auth_url.replace('v2.0', 'v3')
        else:
            # Import auth_token to have keystone_authtoken settings setup.
            importutils.import_module('keystonemiddleware.auth_token')
            self.endpoint = \
                cfg.CONF.keystone_authtoken.www_authenticate_uri.replace(
                    'v2.0', 'v3')

        if self.context and self.context.trust_id:
            # Create a client with the specified trust_id, this
            # populates self.context.auth_token with a trust-scoped token
            self._client = self._v3_client_init()

    @property
    def client(self):
        if not self._client:
            # Create connection to v3 API
            self._client = self._v3_client_init()
        return self._client

    @property
    def admin_client(self):
        if not self._admin_client:
            # Create admin client connection to v3 API
            admin_creds = self._service_admin_creds()
            auth = identity.Password(**admin_creds)
            session = ks_session.Session(auth=auth)
            c = kc_v3.Client(session=session)
            self._admin_client = c
        return self._admin_client

    @property
    def lp_admin_client(self):
        if not self._lp_admin_client:
            # Create lp operator client connection to v3 API
            lp_operator_creds = self._lp_operator_creds()
            auth = identity.Password(**lp_operator_creds)
            session = ks_session.Session(auth=auth)
            c = kc_v3.Client(session=session)
            self._lp_admin_client = c
        return self._lp_admin_client

    def _v3_client_init(self):
        kwargs = {
            'auth_url': self.endpoint,
            'endpoint': self.endpoint
        }
        # Note try trust_id first, as we can't reuse auth_token in that case
        if self.context.trust_id is not None:
            # We got a trust_id, so we use the admin credentials
            # to authenticate with the trust_id so we can use the
            # trust impersonating the trustor user.
            kwargs.update(self._service_admin_creds())
            kwargs['trust_id'] = self.context.trust_id
            kwargs.pop('project_name')
            auth = ka_loading.load_auth_from_conf_options(
                cfg.CONF, 'keystone_authtoken', **kwargs)
        elif self.context.auth_token is not None:
            kwargs['token'] = self.context.auth_token
            kwargs['project_id'] = self.context.project_id
            auth = identity.Token(
                auth_url=kwargs['auth_url'],
                token=kwargs['token'],
                project_id=kwargs['project_id'])
        else:
            LOG.error(_("Keystone v3 API connection failed, no password "
                        "trust or auth_token!"))
            raise exception.AuthorizationFailure()
        session = ks_session.Session(auth=auth)
        client = kc_v3.Client(session=session)
        client.auth_ref = client.session.auth.get_access(client.session)
        # If we are authenticating with a trust set the context auth_token
        # with the trust scoped token
        if 'trust_id' in kwargs:
            # Sanity check
            if not client.auth_ref.trust_scoped:
                LOG.error(_("trust token re-scoping failed!"))
                raise exception.AuthorizationFailure()
            # All OK so update the context with the token
            self.context.auth_token = client.auth_ref.auth_token
            self.context.auth_url = self.endpoint
            self.context.user = client.auth_ref.user_id
            self.context.project_id = client.auth_ref.project_id
            self.context.user_name = client.auth_ref.username

        return client

    def _service_admin_creds(self):
        # Import auth_token to have keystone_authtoken settings setup.
        importutils.import_module('keystonemiddleware.auth_token')
        creds = {
            'username': cfg.CONF.keystone_authtoken.username,
            'password': cfg.CONF.keystone_authtoken.password,
            'auth_url': self.endpoint,
            'project_name': cfg.CONF.keystone_authtoken.project_name,
            'user_domain_name': "Default",
            'project_domain_name': "Default"}
        return creds

    def _lp_operator_creds(self):
        # Get LP Operator creds from config.
        creds = {
            'username': cfg.CONF.worker.lp_operator_user,
            'password': cfg.CONF.worker.lp_operator_password,
            'auth_url': self.endpoint,
            'project_name': cfg.CONF.worker.lp_operator_tenant_name,
            'user_domain_name': "Default",
            'project_domain_name': "Default"}
        return creds

    def create_trust_context(self):
        """Create a trust using the trustor identity in the current context.

        Use the trustee as the solum service user and return a context
        containing the new trust_id.

        If the current context already contains a trust_id, we do nothing
        and return the current context.
        """
        if self.context.trust_id:
            return self.context

        trustee_user_id = self.admin_client.session.auth.get_user_id(
            self.admin_client.session)
        auth_ref = self.client.session.auth.get_access(self.client.session)
        trustor_user_id = auth_ref.user_id
        trustor_project_id = auth_ref.project_id
        roles = cfg.CONF.trusts_delegated_roles
        trust = self.client.trusts.create(trustor_user=trustor_user_id,
                                          trustee_user=trustee_user_id,
                                          project=trustor_project_id,
                                          impersonation=True,
                                          role_names=roles)

        trust_context = context.RequestContext.from_dict(
            self.context.to_dict())
        trust_context.trust_id = trust.id
        return trust_context

    def delete_trust(self, trust_id):
        """Delete the specified trust."""
        try:
            self.client.trusts.delete(trust_id)
        except kc_exception.NotFound:
            pass

    @property
    def auth_token(self):
        auth_ref = self.client.session.auth.get_access(self.client.session)
        return auth_ref.auth_token



# Copyright 2015 - Rackspace Hosting.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log as logging

from solum.common import clients

LOG = logging.getLogger(__name__)


class GlanceClient(object):
    """Glance client wrapper so we can encapsulate logic in one place."""

    def __init__(self, context):
        self.context = context

    def _get_glance_client(self):
        # Create a new non-cached glance client/connection
        return clients.OpenStackClients(self.context).glance()

    def delete_image_by_name(self, image_name):
        try:
            glance = self._get_glance_client()
            imgs = glance.images.list()
            for i in imgs:
                if i.name == image_name:
                    glance.images.delete(i.id)
        except Exception as e:
            LOG.exception(e)

    def delete_image_by_id(self, image_id):
        try:
            self._get_glance_client().images.delete(image_id)
        except Exception as e:
            LOG.exception(e)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_service
----------------------------------

Tests for `solum.common.service` module.
"""

from solum.common import service
from solum import objects
from solum.objects import component as abstract
from solum.tests import base


class TestService(base.BaseTestCase):

    def test_prepare_invokes_object_load(self):
        objects.registry.clear()
        objects.registry.Component
        objects.load()
        service.prepare_service([])
        self.assertTrue(issubclass(objects.registry.Component,
                                   abstract.Component))



# Copyright 2013 - Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""Starter script for the Solum API service."""

import os
import sys

import eventlet
from eventlet import wsgi
from oslo_config import cfg
from oslo_log import log as logging

from solum.api import app as api_app
from solum.common import service
from solum.i18n import _


LOG = logging.getLogger(__name__)


def main():
    eventlet.monkey_patch(socket=True, select=True, time=True)
    service.prepare_service(sys.argv)

    app = api_app.setup_app()

    # Create the WSGI server and start it
    host, port = cfg.CONF.api.host, cfg.CONF.api.port

    LOG.info(_('Starting server in PID %s') % os.getpid())
    LOG.debug("Configuration:")
    logging.setup(cfg.CONF, 'solum')

    if host == '0.0.0.0':
        LOG.info(_('serving on 0.0.0.0:%(port)s, '
                   'view at http://127.0.0.1:%(port)s') %
                 dict(port=port))
    else:
        LOG.info(_('serving on http://%(host)s:%(port)s') %
                 dict(host=host, port=port))

    wsgi.server(eventlet.listen((host, port)), app)



#!/usr/bin/env python
# Copyright 2014 - Rackspace Hosting
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""
For testing trigger workflow
"""

import argparse
import json
import os

import requests
from solumclient import client as solum_client


SOLUM_API_VERSION = '1'


def _get_solum_client():
    args = {}
    args['os_username'] = os.getenv('OS_USERNAME', '')
    args['os_password'] = os.getenv('OS_PASSWORD', '')
    args['os_tenant_name'] = os.getenv('OS_TENANT_NAME', '')
    args['os_auth_url'] = os.getenv('OS_AUTH_URL', '')
    args['solum_url'] = os.getenv('SOLUM_URL', '')

    try:
        client = solum_client.get_client(SOLUM_API_VERSION, **args)
        return client
    except Exception as ex:
        print("Error in getting Solum client: %s" % ex)
        exit(1)


def main(args):
    client = _get_solum_client()
    app = client.apps.find(name_or_id=args.app)
    status_url = 'https://api.github.com/repos/{repo}/statuses/{sha}'.format(
        repo=args.repo, sha=args.sha)
    body_dict = {'sender': {'url': 'https://api.github.com'},
                 'pull_request': {'head': {'sha': args.sha}},
                 'repository': {'statuses_url': status_url}}
    body = json.dumps(body_dict)
    trigger_uri = app.trigger_uri
    if args.workflow:
        trigger_uri = "%s?workflow=%s" % (trigger_uri, args.workflow)
    resp = requests.post(trigger_uri, data=body)
    print('status code is %s' % resp.status_code)


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('app', help="app name/UUID")
    parser.add_argument('--sha', default='xyz', dest='sha',
                        help="the commit sha to clone from")
    parser.add_argument('--repo', default='u/r', dest='repo',
                        help="{user}/{repo}")
    parser.add_argument('--workflow', dest="workflow",
                        help="Workflow to trigger")

    args = parser.parse_args()
    main(args)



# Copyright 2014 - Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sqlalchemy as sa

from solum.objects.sqlalchemy import models as sql
from solum.objects import userlog as abstract


class Userlog(sql.Base, abstract.Userlog):
    """Represent a userlog in sqlalchemy."""

    __tablename__ = 'userlogs'
    __resource__ = 'userlog'
    __table_args__ = sql.table_args()

    id = sa.Column(sa.Integer, primary_key=True, autoincrement=True)
    resource_uuid = sa.Column(sa.String(36), nullable=False)
    project_id = sa.Column(sa.String(36))
    created_at = sa.Column(sa.DateTime)
    location = sa.Column(sa.String(255))
    strategy = sa.Column(sa.String(255))
    strategy_info = sa.Column(sa.String(1024))
    resource_type = sa.Column(sa.String(36))


class UserlogList(abstract.UserlogList):
    """Represent a list of userlogs in sqlalchemy."""

    @classmethod
    def get_all(cls, context):
        return UserlogList(sql.model_query(context, Userlog))

    @classmethod
    def get_all_by_id(cls, context, resource_uuid):
        session = sql.Base.get_session()
        logs = session.query(Userlog).filter_by(project_id=context.project_id)
        logs = logs.filter_by(resource_uuid=resource_uuid)
        return logs.order_by(Userlog.created_at).all()
